@@ -38,6 +38,7 @@
  * @MM_MODEM_CAPABILITY_LTE: Modem has LTE data capability.
  * @MM_MODEM_CAPABILITY_LTE_ADVANCED: Modem has LTE Advanced data capability.
  * @MM_MODEM_CAPABILITY_IRIDIUM: Modem has Iridium capabilities.
+ * @MM_MODEM_CAPABILITY_ANY: Mask specifying all capabilities.
  *
  * Flags describing one or more of the general access technology families that a
  * modem supports.
@@ -50,6 +51,7 @@ typedef enum { /*< underscore_name=mm_modem_capability >*/
     MM_MODEM_CAPABILITY_LTE          = 1 << 3,
     MM_MODEM_CAPABILITY_LTE_ADVANCED = 1 << 4,
     MM_MODEM_CAPABILITY_IRIDIUM      = 1 << 5,
+    MM_MODEM_CAPABILITY_ANY          = 0xFFFFFFFF
 } MMModemCapability;
 
 /**
@@ -128,11 +130,44 @@ typedef enum { /*< underscore_name=mm_modem_state >*/
     MM_MODEM_STATE_CONNECTED     = 11
 } MMModemState;
 
+/**
+ * MMModemStateFailedReason:
+ * @MM_MODEM_STATE_FAILED_REASON_NONE: No error.
+ * @MM_MODEM_STATE_FAILED_REASON_UNKNOWN: Unknown error.
+ * @MM_MODEM_STATE_FAILED_REASON_SIM_MISSING: SIM is required but missing.
+ * @MM_MODEM_STATE_FAILED_REASON_SIM_ERROR: SIM is available, but unusable (e.g. permanently locked).
+ *
+ * Enumeration of possible errors when the modem is in @MM_MODEM_STATE_FAILED.
+ */
+typedef enum { /*< underscore_name=mm_modem_state_failed_reason >*/
+    MM_MODEM_STATE_FAILED_REASON_NONE        = 0,
+    MM_MODEM_STATE_FAILED_REASON_UNKNOWN     = 1,
+    MM_MODEM_STATE_FAILED_REASON_SIM_MISSING = 2,
+    MM_MODEM_STATE_FAILED_REASON_SIM_ERROR   = 3,
+} MMModemStateFailedReason;
+
+/**
+ * MMModemPowerState:
+ * @MM_MODEM_POWER_STATE_UNKNOWN: Unknown power state.
+ * @MM_MODEM_POWER_STATE_OFF: Off.
+ * @MM_MODEM_POWER_STATE_LOW: Low-power mode.
+ * @MM_MODEM_POWER_STATE_ON: Full power mode.
+ *
+ * Power state of the modem.
+ */
+typedef enum { /*< underscore_name=mm_modem_power_state >*/
+    MM_MODEM_POWER_STATE_UNKNOWN = 0,
+    MM_MODEM_POWER_STATE_OFF     = 1,
+    MM_MODEM_POWER_STATE_LOW     = 2,
+    MM_MODEM_POWER_STATE_ON      = 3
+} MMModemPowerState;
+
 /**
  * MMModemStateChangeReason:
  * @MM_MODEM_STATE_CHANGE_REASON_UNKNOWN: Reason unknown or not reportable.
  * @MM_MODEM_STATE_CHANGE_REASON_USER_REQUESTED: State change was requested by an interface user.
  * @MM_MODEM_STATE_CHANGE_REASON_SUSPEND: State change was caused by a system suspend.
+ * @MM_MODEM_STATE_CHANGE_REASON_FAILURE: State change was caused by an unrecoverable error.
  *
  * Enumeration of possible reasons to have changed the modem state.
  */
@@ -140,6 +175,7 @@ typedef enum { /*< underscore_name=mm_modem_state_change_reason >*/
     MM_MODEM_STATE_CHANGE_REASON_UNKNOWN        = 0,
     MM_MODEM_STATE_CHANGE_REASON_USER_REQUESTED = 1,
     MM_MODEM_STATE_CHANGE_REASON_SUSPEND        = 2,
+    MM_MODEM_STATE_CHANGE_REASON_FAILURE        = 3,
 } MMModemStateChangeReason;
 
 /**
@@ -487,6 +523,22 @@ typedef enum { /*< underscore_name=mm_sms_storage >*/
     MM_SMS_STORAGE_TA      = 6,
 } MMSmsStorage;
 
+/**
+ * MMSmsValidityType:
+ * @MM_SMS_VALIDITY_TYPE_UNKNOWN: Validity type unknown.
+ * @MM_SMS_VALIDITY_TYPE_RELATIVE: Relative validity.
+ * @MM_SMS_VALIDITY_TYPE_ABSOLUTE: Absolute validity.
+ * @MM_SMS_VALIDITY_TYPE_ENHANCED: Enhanced validity.
+ *
+ * Type of SMS validity value.
+ */
+typedef enum { /*< underscore_name=mm_sms_validity_type >*/
+    MM_SMS_VALIDITY_TYPE_UNKNOWN  = 0,
+    MM_SMS_VALIDITY_TYPE_RELATIVE = 1,
+    MM_SMS_VALIDITY_TYPE_ABSOLUTE = 2,
+    MM_SMS_VALIDITY_TYPE_ENHANCED = 3,
+} MMSmsValidityType;
+
 /**
  * MMModemLocationSource:
  * @MM_MODEM_LOCATION_SOURCE_NONE: None.
@@ -539,18 +591,20 @@ typedef enum { /*< underscore_name=mm_bearer_ip_method >*/
 
 /**
  * MMBearerIpFamily:
- * @MM_BEARER_IP_FAMILY_UNKNOWN: Unknown.
+ * @MM_BEARER_IP_FAMILY_NONE: None or unknown.
  * @MM_BEARER_IP_FAMILY_IPV4: IPv4.
  * @MM_BEARER_IP_FAMILY_IPV6: IPv6.
  * @MM_BEARER_IP_FAMILY_IPV4V6: IPv4 and IPv6.
+ * @MM_BEARER_IP_FAMILY_ANY: Mask specifying all IP families.
  *
  * Type of IP family to be used in a given Bearer.
  */
 typedef enum { /*< underscore_name=mm_bearer_ip_family >*/
-    MM_BEARER_IP_FAMILY_UNKNOWN = 0,
-    MM_BEARER_IP_FAMILY_IPV4    = 4,
-    MM_BEARER_IP_FAMILY_IPV6    = 6,
-    MM_BEARER_IP_FAMILY_IPV4V6  = 10
+    MM_BEARER_IP_FAMILY_NONE    = 0,
+    MM_BEARER_IP_FAMILY_IPV4    = 1 << 0,
+    MM_BEARER_IP_FAMILY_IPV6    = 1 << 1,
+    MM_BEARER_IP_FAMILY_IPV4V6  = 1 << 2,
+    MM_BEARER_IP_FAMILY_ANY     = 0xFFFFFFFF
 } MMBearerIpFamily;
 
 /**
@@ -594,6 +648,7 @@ typedef enum { /*< underscore_name=mm_modem_cdma_registration_state >*/
 
 /**
  * MMModemCdmaActivationState:
+ * @MM_MODEM_CDMA_ACTIVATION_STATE_UNKNOWN: Unknown activation state.
  * @MM_MODEM_CDMA_ACTIVATION_STATE_NOT_ACTIVATED: Device is not activated
  * @MM_MODEM_CDMA_ACTIVATION_STATE_ACTIVATING: Device is activating
  * @MM_MODEM_CDMA_ACTIVATION_STATE_PARTIALLY_ACTIVATED: Device is partially activated; carrier-specific steps required to continue.
@@ -602,10 +657,11 @@ typedef enum { /*< underscore_name=mm_modem_cdma_registration_state >*/
  * Activation state of a CDMA modem.
  */
 typedef enum { /*< underscore_name=mm_modem_cdma_activation_state >*/
-    MM_MODEM_CDMA_ACTIVATION_STATE_NOT_ACTIVATED       = 0,
-    MM_MODEM_CDMA_ACTIVATION_STATE_ACTIVATING          = 1,
-    MM_MODEM_CDMA_ACTIVATION_STATE_PARTIALLY_ACTIVATED = 2,
-    MM_MODEM_CDMA_ACTIVATION_STATE_ACTIVATED           = 3,
+    MM_MODEM_CDMA_ACTIVATION_STATE_UNKNOWN             = 0,
+    MM_MODEM_CDMA_ACTIVATION_STATE_NOT_ACTIVATED       = 1,
+    MM_MODEM_CDMA_ACTIVATION_STATE_ACTIVATING          = 2,
+    MM_MODEM_CDMA_ACTIVATION_STATE_PARTIALLY_ACTIVATED = 3,
+    MM_MODEM_CDMA_ACTIVATION_STATE_ACTIVATED           = 4,
 } MMModemCdmaActivationState;
 
 /**
