@@ -18,6 +18,7 @@
 #include <stdlib.h>
 
 #include "mm-errors-types.h"
+#include "mm-enums-types.h"
 #include "mm-common-helpers.h"
 #include "mm-sms-properties.h"
 
@@ -48,10 +49,9 @@ struct _MMSmsPropertiesPrivate {
     GByteArray *data;
     gchar *number;
     gchar *smsc;
-    gboolean validity_set;
-    guint validity;
-    gboolean class_set;
-    guint class;
+    MMSmsValidityType validity_type;
+    guint validity_relative;
+    gint class;
     gboolean delivery_report_request_set;
     gboolean delivery_report_request;
 };
@@ -264,36 +264,53 @@ mm_sms_properties_get_smsc (MMSmsProperties *self)
 /*****************************************************************************/
 
 /**
- * mm_sms_properties_set_validity:
+ * mm_sms_properties_set_validity_relative:
  * @self: A #MMSmsProperties.
- * @validity: The validity.
+ * @validity: The validity of %MM_SMS_VALIDITY_TYPE_RELATIVE type.
  *
- * Sets the validity time of the SMS.
+ * Sets the relative validity time of the SMS.
  */
 void
-mm_sms_properties_set_validity (MMSmsProperties *self,
-                                guint validity)
+mm_sms_properties_set_validity_relative (MMSmsProperties *self,
+                                         guint validity)
 {
     g_return_if_fail (MM_IS_SMS_PROPERTIES (self));
 
-    self->priv->validity_set = TRUE;
-    self->priv->validity = validity;
+    self->priv->validity_type = MM_SMS_VALIDITY_TYPE_RELATIVE;
+    self->priv->validity_relative = validity;
 }
 
 /**
- * mm_sms_properties_get_validity:
+ * mm_sms_properties_get_validity_type:
  * @self: A #MMSmsProperties.
  *
- * Gets the validity time of the SMS.
+ * Gets the relative validity type the SMS.
+ *
+ * Returns: a #MMSmsValidityType.
+ */
+MMSmsValidityType
+mm_sms_properties_get_validity_type (MMSmsProperties *self)
+{
+    g_return_val_if_fail (MM_IS_SMS_PROPERTIES (self), 0);
+
+    return self->priv->validity_type;
+}
+
+/**
+ * mm_sms_properties_get_validity_relative:
+ * @self: A #MMSmsProperties.
+ *
+ * Gets the relative validity time of the SMS.
  *
  * Returns: the validity time or 0 if unknown.
  */
 guint
-mm_sms_properties_get_validity (MMSmsProperties *self)
+mm_sms_properties_get_validity_relative (MMSmsProperties *self)
 {
     g_return_val_if_fail (MM_IS_SMS_PROPERTIES (self), 0);
+    g_return_val_if_fail (self->priv->validity_type == MM_SMS_VALIDITY_TYPE_RELATIVE, 0);
 
-    return self->priv->validity;
+    return self->priv->validity_relative;
 }
 
 /*****************************************************************************/
@@ -301,17 +318,17 @@ mm_sms_properties_get_validity (MMSmsProperties *self)
 /**
  * mm_sms_properties_set_class:
  * @self: A #MMSmsProperties.
- * @class: The message class.
+ * @class: The message class (0..3), or -1 for invalid/unset class.
  *
  * Sets the 3GPP message class of the SMS.
  */
 void
 mm_sms_properties_set_class (MMSmsProperties *self,
-                             guint class)
+                             gint class)
 {
     g_return_if_fail (MM_IS_SMS_PROPERTIES (self));
+    g_return_if_fail (class >= -1 && class <= 3);
 
-    self->priv->class_set = TRUE;
     self->priv->class = class;
 }
 
@@ -321,12 +338,12 @@ mm_sms_properties_set_class (MMSmsProperties *self,
  *
  * Gets the 3GPP message class of the SMS.
  *
- * Returns: the message class.
+ * Returns: the message class, or -1 for invalid/unset class.
  */
-guint
+gint
 mm_sms_properties_get_class (MMSmsProperties *self)
 {
-    g_return_val_if_fail (MM_IS_SMS_PROPERTIES (self), 0);
+    g_return_val_if_fail (MM_IS_SMS_PROPERTIES (self), -1);
 
     return self->priv->class;
 }
@@ -411,17 +428,17 @@ mm_sms_properties_get_dictionary (MMSmsProperties *self)
                                PROPERTY_SMSC,
                                g_variant_new_string (self->priv->smsc));
 
-    if (self->priv->validity_set)
+    if (self->priv->validity_type == MM_SMS_VALIDITY_TYPE_RELATIVE)
         g_variant_builder_add (&builder,
                                "{sv}",
                                PROPERTY_VALIDITY,
-                               g_variant_new_uint32 (self->priv->validity));
+                               g_variant_new ("(uv)", MM_SMS_VALIDITY_TYPE_RELATIVE, g_variant_new_uint32 (self->priv->validity_relative)));
 
-    if (self->priv->class_set)
+    if (self->priv->class >= 0)
         g_variant_builder_add (&builder,
                                "{sv}",
                                PROPERTY_CLASS,
-                               g_variant_new_uint32 (self->priv->class));
+                               g_variant_new_int32 (self->priv->class));
 
     if (self->priv->delivery_report_request_set)
         g_variant_builder_add (&builder,
@@ -497,14 +514,16 @@ consume_string (MMSmsProperties *self,
             return FALSE;
         }
 
-        mm_sms_properties_set_validity (self, n);
+        mm_sms_properties_set_validity_relative (self, n);
     } else if (g_str_equal (key, PROPERTY_CLASS)) {
-        GError *inner_error = NULL;
-        guint n;
+        gint n = 0;
 
-        n = parse_uint (value, &inner_error);
-        if (inner_error) {
-            g_propagate_error (error, inner_error);
+        if (!mm_get_int_from_str (value, &n)) {
+            g_set_error (error,
+                         MM_CORE_ERROR,
+                         MM_CORE_ERROR_INVALID_ARGS,
+                         "Invalid properties string, cannot parse '%s' as int",
+                         value);
             return FALSE;
         }
 
@@ -608,14 +627,23 @@ consume_variant (MMSmsProperties *properties,
         mm_sms_properties_set_smsc (
             properties,
             g_variant_get_string (value, NULL));
-    else if (g_str_equal (key, PROPERTY_VALIDITY))
-        mm_sms_properties_set_validity (
-            properties,
-            g_variant_get_uint32 (value));
-    else if (g_str_equal (key, PROPERTY_CLASS))
+    else if (g_str_equal (key, PROPERTY_VALIDITY)) {
+        guint type;
+        GVariant *val;
+
+        g_variant_get (value, "(uv)", &type, &val);
+        if (type == MM_SMS_VALIDITY_TYPE_RELATIVE) {
+            mm_sms_properties_set_validity_relative (
+                properties,
+                g_variant_get_uint32 (val));
+        } else if (type != MM_SMS_VALIDITY_TYPE_UNKNOWN)
+            g_warning ("SMS validity type '%s' not supported yet", 
+                       mm_sms_validity_type_get_string (type));
+        g_variant_unref (val);
+    } else if (g_str_equal (key, PROPERTY_CLASS))
         mm_sms_properties_set_class (
             properties,
-            g_variant_get_uint32 (value));
+            g_variant_get_int32 (value));
     else if (g_str_equal (key, PROPERTY_DELIVERY_REPORT_REQUEST))
         mm_sms_properties_set_delivery_report_request (
             properties,
@@ -718,6 +746,8 @@ mm_sms_properties_init (MMSmsProperties *self)
     self->priv = G_TYPE_INSTANCE_GET_PRIVATE ((self),
                                               MM_TYPE_SMS_PROPERTIES,
                                               MMSmsPropertiesPrivate);
+    self->priv->validity_type = MM_SMS_VALIDITY_TYPE_UNKNOWN;
+    self->priv->class = -1;
 }
 
 static void
