@@ -40,12 +40,13 @@ static void iface_modem_init (MMIfaceModem *iface);
 static void iface_modem_3gpp_init (MMIfaceModem3gpp *iface);
 static void iface_modem_time_init (MMIfaceModemTime *iface);
 
+static MMIfaceModem *iface_modem_parent;
 static MMIfaceModem3gpp *iface_modem_3gpp_parent;
 
 G_DEFINE_TYPE_EXTENDED (MMBroadbandModemIcera, mm_broadband_modem_icera, MM_TYPE_BROADBAND_MODEM, 0,
                         G_IMPLEMENT_INTERFACE (MM_TYPE_IFACE_MODEM, iface_modem_init)
                         G_IMPLEMENT_INTERFACE (MM_TYPE_IFACE_MODEM_3GPP, iface_modem_3gpp_init)
-                        G_IMPLEMENT_INTERFACE (MM_TYPE_IFACE_MODEM_TIME, iface_modem_time_init));
+                        G_IMPLEMENT_INTERFACE (MM_TYPE_IFACE_MODEM_TIME, iface_modem_time_init))
 
 enum {
     PROP_0,
@@ -66,11 +67,93 @@ struct _MMBroadbandModemIceraPrivate {
     MMModemAccessTechnology last_act;
 };
 
+/*****************************************************************************/
+/* Load supported modes (Modem interface) */
+
+static GArray *
+load_supported_modes_finish (MMIfaceModem *self,
+                             GAsyncResult *res,
+                             GError **error)
+{
+    if (g_simple_async_result_propagate_error (G_SIMPLE_ASYNC_RESULT (res), error))
+        return NULL;
+
+    return g_array_ref (g_simple_async_result_get_op_res_gpointer (G_SIMPLE_ASYNC_RESULT (res)));
+}
+
+static void
+parent_load_supported_modes_ready (MMIfaceModem *self,
+                                   GAsyncResult *res,
+                                   GSimpleAsyncResult *simple)
+{
+    GError *error = NULL;
+    GArray *all;
+    GArray *combinations;
+    GArray *filtered;
+    MMModemModeCombination mode;
+
+    all = iface_modem_parent->load_supported_modes_finish (self, res, &error);
+    if (!all) {
+        g_simple_async_result_take_error (simple, error);
+        g_simple_async_result_complete (simple);
+        g_object_unref (simple);
+        return;
+    }
+
+    /* Build list of combinations */
+    combinations = g_array_sized_new (FALSE, FALSE, sizeof (MMModemModeCombination), 5);
+
+    /* 2G only */
+    mode.allowed = MM_MODEM_MODE_2G;
+    mode.preferred = MM_MODEM_MODE_NONE;
+    g_array_append_val (combinations, mode);
+    /* 3G only */
+    mode.allowed = MM_MODEM_MODE_3G;
+    mode.preferred = MM_MODEM_MODE_NONE;
+    g_array_append_val (combinations, mode);
+    /* 2G and 3G */
+    mode.allowed = (MM_MODEM_MODE_2G | MM_MODEM_MODE_3G);
+    mode.preferred = MM_MODEM_MODE_NONE;
+    g_array_append_val (combinations, mode);
+    /* 2G and 3G, 2G preferred */
+    mode.allowed = (MM_MODEM_MODE_2G | MM_MODEM_MODE_3G);
+    mode.preferred = MM_MODEM_MODE_2G;
+    g_array_append_val (combinations, mode);
+    /* 2G and 3G, 3G preferred */
+    mode.allowed = (MM_MODEM_MODE_2G | MM_MODEM_MODE_3G);
+    mode.preferred = MM_MODEM_MODE_3G;
+    g_array_append_val (combinations, mode);
+
+    /* Filter out those unsupported modes */
+    filtered = mm_filter_supported_modes (all, combinations);
+    g_array_unref (all);
+    g_array_unref (combinations);
+
+    g_simple_async_result_set_op_res_gpointer (simple, filtered, (GDestroyNotify) g_array_unref);
+    g_simple_async_result_complete (simple);
+    g_object_unref (simple);
+}
+
+static void
+load_supported_modes (MMIfaceModem *self,
+                      GAsyncReadyCallback callback,
+                      gpointer user_data)
+{
+    /* Run parent's loading */
+    iface_modem_parent->load_supported_modes (
+        MM_IFACE_MODEM (self),
+        (GAsyncReadyCallback)parent_load_supported_modes_ready,
+        g_simple_async_result_new (G_OBJECT (self),
+                                   callback,
+                                   user_data,
+                                   load_supported_modes));
+}
+
 /*****************************************************************************/
 /* Load initial allowed/preferred modes (Modem interface) */
 
 static gboolean
-modem_load_allowed_modes_finish (MMIfaceModem *self,
+modem_load_current_modes_finish (MMIfaceModem *self,
                                  GAsyncResult *res,
                                  MMModemMode *allowed,
                                  MMModemMode *preferred,
@@ -129,7 +212,7 @@ modem_load_allowed_modes_finish (MMIfaceModem *self,
 }
 
 static void
-modem_load_allowed_modes (MMIfaceModem *self,
+modem_load_current_modes (MMIfaceModem *self,
                           GAsyncReadyCallback callback,
                           gpointer user_data)
 {
@@ -145,7 +228,7 @@ modem_load_allowed_modes (MMIfaceModem *self,
 /* Set allowed modes (Modem interface) */
 
 static gboolean
-modem_set_allowed_modes_finish (MMIfaceModem *self,
+modem_set_current_modes_finish (MMIfaceModem *self,
                                 GAsyncResult *res,
                                 GError **error)
 {
@@ -170,7 +253,7 @@ allowed_mode_update_ready (MMBaseModem *self,
 }
 
 static void
-modem_set_allowed_modes (MMIfaceModem *self,
+modem_set_current_modes (MMIfaceModem *self,
                          MMModemMode allowed,
                          MMModemMode preferred,
                          GAsyncReadyCallback callback,
@@ -183,7 +266,7 @@ modem_set_allowed_modes (MMIfaceModem *self,
     result = g_simple_async_result_new (G_OBJECT (self),
                                         callback,
                                         user_data,
-                                        modem_set_allowed_modes);
+                                        modem_set_current_modes);
 
     /*
      * The core has checked the following:
@@ -201,7 +284,8 @@ modem_set_allowed_modes (MMIfaceModem *self,
             icera_mode = 3;
         else /* none preferred, so AUTO */
             icera_mode = 5;
-    }
+    } else if (allowed == MM_MODEM_MODE_ANY && preferred == MM_MODEM_MODE_NONE)
+        icera_mode = 5;
 
     if (icera_mode < 0) {
         gchar *allowed_str;
@@ -764,27 +848,79 @@ modem_create_bearer (MMIfaceModem *self,
                                         user_data,
                                         modem_create_bearer);
 
-    /* If data port is AT create a generic bearer */
-    if (MM_IS_AT_SERIAL_PORT (mm_base_modem_peek_best_data_port (MM_BASE_MODEM (self)))) {
-        mm_broadband_bearer_new (
+    /* If we get a NET port, create Icera bearer */
+    if (mm_base_modem_peek_best_data_port (MM_BASE_MODEM (self), MM_PORT_TYPE_NET)) {
+        mm_broadband_bearer_icera_new (
             MM_BROADBAND_MODEM (self),
+            MM_BROADBAND_MODEM_ICERA (self)->priv->default_ip_method,
             properties,
             NULL, /* cancellable */
-            (GAsyncReadyCallback)broadband_bearer_new_ready,
+            (GAsyncReadyCallback)broadband_bearer_icera_new_ready,
             result);
         return;
     }
 
-    /* Otherwise create a Icera bearer */
-    mm_broadband_bearer_icera_new (
+    /* Otherwise, plain generic broadband bearer */
+    mm_broadband_bearer_new (
         MM_BROADBAND_MODEM (self),
-        MM_BROADBAND_MODEM_ICERA (self)->priv->default_ip_method,
         properties,
         NULL, /* cancellable */
-        (GAsyncReadyCallback)broadband_bearer_icera_new_ready,
+        (GAsyncReadyCallback)broadband_bearer_new_ready,
         result);
 }
 
+/*****************************************************************************/
+/* Modem power up (Modem interface) */
+
+static gboolean
+modem_power_up_finish (MMIfaceModem *self,
+                       GAsyncResult *res,
+                       GError **error)
+{
+    return !g_simple_async_result_propagate_error (G_SIMPLE_ASYNC_RESULT (res), error);
+}
+
+static void
+cfun_enable_ready (MMBaseModem *self,
+                   GAsyncResult *res,
+                   GSimpleAsyncResult *simple)
+{
+    GError *error = NULL;
+
+    if (!mm_base_modem_at_command_finish (MM_BASE_MODEM (self), res, &error)) {
+        /* Ignore all errors except NOT_ALLOWED, which means Airplane Mode */
+        if (g_error_matches (error,
+                             MM_MOBILE_EQUIPMENT_ERROR,
+                             MM_MOBILE_EQUIPMENT_ERROR_NOT_ALLOWED))
+            g_simple_async_result_take_error (simple, error);
+        else
+            g_error_free (error);
+    }
+
+    g_simple_async_result_complete (simple);
+    g_object_unref (simple);
+}
+
+static void
+modem_power_up (MMIfaceModem *self,
+                GAsyncReadyCallback callback,
+                gpointer user_data)
+{
+    GSimpleAsyncResult *result;
+
+    result = g_simple_async_result_new (G_OBJECT (self),
+                                        callback,
+                                        user_data,
+                                        modem_power_up);
+
+    mm_base_modem_at_command (MM_BASE_MODEM (self),
+                              "+CFUN=1",
+                              10,
+                              FALSE,
+                              (GAsyncReadyCallback)cfun_enable_ready,
+                              result);
+}
+
 /*****************************************************************************/
 /* Modem power down (Modem interface) */
 
@@ -1012,7 +1148,8 @@ parse_bands (const gchar *response, guint32 *out_len)
 
 typedef struct {
     MMBaseModemAtCommand *cmds;
-    GSList *bands;
+    GSList *check_bands;
+    GSList *enabled_bands;
     guint32 idx;
 } SupportedBandsContext;
 
@@ -1024,7 +1161,8 @@ supported_bands_context_free (SupportedBandsContext *ctx)
     for (i = 0; ctx->cmds[i].command; i++)
         g_free (ctx->cmds[i].command);
     g_free (ctx->cmds);
-    g_slist_free_full (ctx->bands, (GDestroyNotify) band_free);
+    g_slist_free_full (ctx->check_bands, (GDestroyNotify) band_free);
+    g_slist_free_full (ctx->enabled_bands, (GDestroyNotify) band_free);
     g_free (ctx);
 }
 
@@ -1055,12 +1193,21 @@ load_supported_bands_ready (MMBaseModem *self,
         g_simple_async_result_take_error (simple, error);
     else {
         bands = g_array_sized_new (FALSE, FALSE, sizeof (MMModemBand), ctx->idx);
-        for (iter = ctx->bands; iter; iter = g_slist_next (iter)) {
+
+        /* Add already enabled bands */
+        for (iter = ctx->enabled_bands; iter; iter = g_slist_next (iter)) {
+            Band *b = iter->data;
+
+            g_array_prepend_val (bands, b->band);
+        }
+
+        /* Add any checked bands that are supported */
+        for (iter = ctx->check_bands; iter; iter = g_slist_next (iter)) {
             Band *b = iter->data;
 
             /* 'enabled' here really means supported/unsupported */
             if (b->enabled)
-                g_array_append_val (bands, b->band);
+                g_array_prepend_val (bands, b->band);
         }
 
         g_simple_async_result_set_op_res_gpointer (simple,
@@ -1082,7 +1229,7 @@ load_supported_bands_response_processor (MMBaseModem *self,
                                          GError **result_error)
 {
     SupportedBandsContext *ctx = context;
-    Band *b = g_slist_nth_data (ctx->bands, ctx->idx++);
+    Band *b = g_slist_nth_data (ctx->check_bands, ctx->idx++);
 
     /* If there was no error setting the band, that band is supported.  We
      * abuse the 'enabled' item to mean supported/unsupported.
@@ -1094,15 +1241,15 @@ load_supported_bands_response_processor (MMBaseModem *self,
 }
 
 static void
-load_supported_bands_get_bands_ready (MMIfaceModem *self,
-                                      GAsyncResult *res,
-                                      GSimpleAsyncResult *operation_result)
+load_supported_bands_get_current_bands_ready (MMIfaceModem *self,
+                                              GAsyncResult *res,
+                                              GSimpleAsyncResult *operation_result)
 {
     SupportedBandsContext *ctx;
     const gchar *response;
     GError *error = NULL;
-    GSList *iter;
-    guint32 len = 0, i;
+    GSList *iter, *new;
+    guint32 len = 0, i = 0;
 
     response = mm_base_modem_at_command_finish (MM_BASE_MODEM (self), res, &error);
     if (!response) {
@@ -1118,18 +1265,28 @@ load_supported_bands_get_bands_ready (MMIfaceModem *self,
     /* For each reported band, build up an AT command to set that band
      * to its current enabled/disabled state.
      */
-    ctx->bands = parse_bands (response, &len);
+    iter = ctx->check_bands = parse_bands (response, &len);
     ctx->cmds = g_new0 (MMBaseModemAtCommand, len + 1);
 
-    for (iter = ctx->bands, i = 0; iter; iter = g_slist_next (iter), i++) {
+    while (iter) {
         Band *b = iter->data;
 
-        ctx->cmds[i].command = g_strdup_printf ("%%IPBM=\"%s\",%c",
-                                                b->name,
-                                                b->enabled ? '1' : '0');
-        ctx->cmds[i].timeout = 3;
-        ctx->cmds[i].allow_cached = FALSE;
-        ctx->cmds[i].response_processor = load_supported_bands_response_processor;
+        if (b->enabled || b->band == MM_MODEM_BAND_ANY) {
+            /* Move known-supported band to the enabled list */
+            new = g_slist_next (iter);
+            ctx->check_bands = g_slist_remove_link (ctx->check_bands, iter);
+            ctx->enabled_bands = g_slist_prepend (ctx->enabled_bands, iter->data);
+            g_slist_free (iter);
+            iter = new;
+        } else {
+            /* Check support for disabled band */
+            ctx->cmds[i].command = g_strdup_printf ("%%IPBM=\"%s\",0", b->name);
+            ctx->cmds[i].timeout = 10;
+            ctx->cmds[i].allow_cached = FALSE;
+            ctx->cmds[i].response_processor = load_supported_bands_response_processor;
+            i++;
+            iter = g_slist_next (iter);
+        }
     }
 
     mm_base_modem_at_sequence (MM_BASE_MODEM (self),
@@ -1155,7 +1312,7 @@ modem_load_supported_bands (MMIfaceModem *self,
         "%IPBM?",
         3,
         FALSE,
-        (GAsyncReadyCallback)load_supported_bands_get_bands_ready,
+        (GAsyncReadyCallback)load_supported_bands_get_current_bands_ready,
         g_simple_async_result_new (G_OBJECT (self),
                                    callback,
                                    user_data,
@@ -1232,14 +1389,14 @@ modem_load_current_bands (MMIfaceModem *self,
 }
 
 /*****************************************************************************/
-/* Set bands (Modem interface) */
+/* Set current bands (Modem interface) */
 
 typedef struct {
     GSimpleAsyncResult *result;
     guint bandbits;
     guint enablebits;
     guint disablebits;
-} SetBandsContext;
+} SetCurrentBandsContext;
 
 /*
  * The modem's band-setting command (%IPBM=) enables or disables one
@@ -1249,34 +1406,34 @@ typedef struct {
  * disable any removed bands.
  */
 static gboolean
-modem_set_bands_finish (MMIfaceModem *self,
+modem_set_current_bands_finish (MMIfaceModem *self,
                         GAsyncResult *res,
                         GError **error)
 {
     return !g_simple_async_result_propagate_error (G_SIMPLE_ASYNC_RESULT (res), error);
 }
 
-static void set_one_band (MMIfaceModem *self, SetBandsContext *ctx);
+static void set_one_band (MMIfaceModem *self, SetCurrentBandsContext *ctx);
 
 static void
-set_bands_context_complete_and_free (SetBandsContext *ctx)
+set_current_bands_context_complete_and_free (SetCurrentBandsContext *ctx)
 {
     g_simple_async_result_complete (ctx->result);
     g_object_unref (ctx->result);
-    g_free (ctx);
+    g_slice_free (SetCurrentBandsContext, ctx);
 }
 
 static void
-set_bands_next (MMIfaceModem *self,
-                GAsyncResult *res,
-                SetBandsContext *ctx)
+set_current_bands_next (MMIfaceModem *self,
+                        GAsyncResult *res,
+                        SetCurrentBandsContext *ctx)
 {
     GError *error = NULL;
 
     if (!mm_base_modem_at_command_finish (MM_BASE_MODEM (self), res, &error)) {
-        mm_dbg ("Couldn't set bands: '%s'", error->message);
+        mm_dbg ("Couldn't set current bands: '%s'", error->message);
         g_simple_async_result_take_error (ctx->result, error);
-        set_bands_context_complete_and_free (ctx);
+        set_current_bands_context_complete_and_free (ctx);
         return;
     }
 
@@ -1285,7 +1442,7 @@ set_bands_next (MMIfaceModem *self,
 
 static void
 set_one_band (MMIfaceModem *self,
-              SetBandsContext *ctx)
+              SetCurrentBandsContext *ctx)
 {
     guint enable, band;
     gchar *command;
@@ -1300,7 +1457,7 @@ set_one_band (MMIfaceModem *self,
     if (band == 0) {
         /* Both enabling and disabling are done */
         g_simple_async_result_set_op_res_gboolean (ctx->result, TRUE);
-        set_bands_context_complete_and_free (ctx);
+        set_current_bands_context_complete_and_free (ctx);
         return;
     }
 
@@ -1324,7 +1481,7 @@ set_one_band (MMIfaceModem *self,
         command,
         10,
         FALSE,
-        (GAsyncReadyCallback)set_bands_next,
+        (GAsyncReadyCallback)set_current_bands_next,
         ctx);
     g_free (command);
 }
@@ -1351,9 +1508,9 @@ band_array_to_bandbits (GArray *bands)
 }
 
 static void
-set_bands_got_current_bands (MMIfaceModem *self,
-                             GAsyncResult *res,
-                             SetBandsContext *ctx)
+set_current_bands_got_current_bands (MMIfaceModem *self,
+                                     GAsyncResult *res,
+                                     SetCurrentBandsContext *ctx)
 {
     GArray *bands;
     GError *error = NULL;
@@ -1362,7 +1519,7 @@ set_bands_got_current_bands (MMIfaceModem *self,
     bands = modem_load_current_bands_finish (self, res, &error);
     if (!bands) {
         g_simple_async_result_take_error (ctx->result, error);
-        set_bands_context_complete_and_free (ctx);
+        set_current_bands_context_complete_and_free (ctx);
         return;
     }
 
@@ -1374,18 +1531,18 @@ set_bands_got_current_bands (MMIfaceModem *self,
 }
 
 static void
-modem_set_bands (MMIfaceModem *self,
-                 GArray *bands_array,
-                 GAsyncReadyCallback callback,
-                 gpointer user_data)
+modem_set_current_bands (MMIfaceModem *self,
+                         GArray *bands_array,
+                         GAsyncReadyCallback callback,
+                         gpointer user_data)
 {
-    SetBandsContext *ctx;
+    SetCurrentBandsContext *ctx;
 
-    ctx = g_new0 (SetBandsContext, 1);
+    ctx = g_slice_new0 (SetCurrentBandsContext);
     ctx->result = g_simple_async_result_new (G_OBJECT (self),
                                              callback,
                                              user_data,
-                                             modem_set_bands);
+                                             modem_set_current_bands);
     ctx->bandbits = band_array_to_bandbits (bands_array);
 
     /*
@@ -1399,7 +1556,7 @@ modem_set_bands (MMIfaceModem *self,
     }
 
     modem_load_current_bands (self,
-                              (GAsyncReadyCallback)set_bands_got_current_bands,
+                              (GAsyncReadyCallback)set_current_bands_got_current_bands,
                               ctx);
 }
 
@@ -1420,7 +1577,9 @@ parse_tlts_query_reply (const gchar *response,
     gint second;
     gchar sign;
     gint offset;
+    GDateTime *utc, *adjusted;
 
+    /* TLTS reports UTC time with the TZ offset to *local* time */
     response = mm_strip_tag (response, "*TLTS: ");
     if (sscanf (response,
                 "\"%02d/%02d/%02d,%02d:%02d:%02d%c%02d\"",
@@ -1431,42 +1590,70 @@ parse_tlts_query_reply (const gchar *response,
                 &minute,
                 &second,
                 &sign,
-                &offset) == 8) {
-        /* Offset comes in 15-min intervals */
-        offset *= 15;
-        /* Apply sign to offset */
-        if (sign == '-')
-            offset *= -1;
-
-        /* If asked for it, build timezone information */
-        if (tz) {
-            *tz = mm_network_timezone_new ();
-            mm_network_timezone_set_offset (*tz, offset);
-        }
-
-        if (iso8601) {
-            /* Icera modems only report a 2-digit year, while ISO-8601 requires
-             * a 4-digit year.  Assume 2000.
-             */
-            if (year < 100)
-                year += 2000;
-
-            /* don't give tz info in the date/time string, we have another
-             * property for that */
-            *iso8601 = g_strdup_printf ("%04d/%02d/%02d %02d:%02d:%02d",
-                                        year, month, day,
-                                        hour, minute, second);
-        }
-
-        return TRUE;
+                &offset) != 8) {
+        g_set_error (error,
+                     MM_CORE_ERROR,
+                     MM_CORE_ERROR_FAILED,
+                     "Unknown *TLTS response: %s",
+                     response);
+        return FALSE;
     }
 
-    g_set_error (error,
-                 MM_CORE_ERROR,
-                 MM_CORE_ERROR_FAILED,
-                 "Unknown *TLTS response: %s",
-                 response);
-    return FALSE;
+    /* Icera modems only report a 2-digit year, while ISO-8601 requires
+     * a 4-digit year.  Assume 2000.
+     */
+    if (year < 100)
+        year += 2000;
+
+    /* Offset comes in 15-min units */
+    offset *= 15;
+    /* Apply sign to offset;  */
+    if (sign == '-')
+        offset *= -1;
+
+    utc = g_date_time_new_utc (year, month, day, hour, minute, second);
+    if (!utc) {
+        g_set_error (error,
+                     MM_CORE_ERROR,
+                     MM_CORE_ERROR_FAILED,
+                     "Invalid *TLTS date/time: %s",
+                     response);
+        return FALSE;
+    }
+
+    /* Convert UTC time to local time by adjusting by the timezone offset */
+    adjusted = g_date_time_add_minutes (utc, offset);
+    g_date_time_unref (utc);
+    if (!adjusted) {
+        g_set_error (error,
+                     MM_CORE_ERROR,
+                     MM_CORE_ERROR_FAILED,
+                     "Failed to convert modem time to local time (offset %d)",
+                     offset);
+        return FALSE;
+    }
+
+    /* Convert offset from minutes-to-UTC to minutes-from-UTC */
+    offset *= -1;
+
+    if (tz) {
+        *tz = mm_network_timezone_new ();
+        mm_network_timezone_set_offset (*tz, offset);
+    }
+
+    if (iso8601) {
+        *iso8601 = mm_new_iso8601_time (g_date_time_get_year (adjusted),
+                                        g_date_time_get_month (adjusted),
+                                        g_date_time_get_day_of_month (adjusted),
+                                        g_date_time_get_hour (adjusted),
+                                        g_date_time_get_minute (adjusted),
+                                        g_date_time_get_second (adjusted),
+                                        TRUE,
+                                        offset);
+    }
+
+    g_date_time_unref (adjusted);
+    return TRUE;
 }
 
 static MMNetworkTimezone *
@@ -1666,10 +1853,14 @@ finalize (GObject *object)
 static void
 iface_modem_init (MMIfaceModem *iface)
 {
-    iface->load_allowed_modes = modem_load_allowed_modes;
-    iface->load_allowed_modes_finish = modem_load_allowed_modes_finish;
-    iface->set_allowed_modes = modem_set_allowed_modes;
-    iface->set_allowed_modes_finish = modem_set_allowed_modes_finish;
+    iface_modem_parent = g_type_interface_peek_parent (iface);
+
+    iface->load_supported_modes = load_supported_modes;
+    iface->load_supported_modes_finish = load_supported_modes_finish;
+    iface->load_current_modes = modem_load_current_modes;
+    iface->load_current_modes_finish = modem_load_current_modes_finish;
+    iface->set_current_modes = modem_set_current_modes;
+    iface->set_current_modes_finish = modem_set_current_modes_finish;
     iface->load_access_technologies = modem_load_access_technologies;
     iface->load_access_technologies_finish = modem_load_access_technologies_finish;
     iface->load_unlock_retries = modem_load_unlock_retries;
@@ -1678,14 +1869,16 @@ iface_modem_init (MMIfaceModem *iface)
     iface->load_supported_bands_finish = modem_load_supported_bands_finish;
     iface->load_current_bands = modem_load_current_bands;
     iface->load_current_bands_finish = modem_load_current_bands_finish;
+    iface->modem_power_up = modem_power_up;
+    iface->modem_power_up_finish = modem_power_up_finish;
     /* Note: don't implement modem_init_power_down, as CFUN=4 here may take
      * looong to reply */
     iface->modem_power_down = modem_power_down;
     iface->modem_power_down_finish = modem_power_down_finish;
     iface->reset = modem_reset;
     iface->reset_finish = modem_reset_finish;
-    iface->set_bands = modem_set_bands;
-    iface->set_bands_finish = modem_set_bands_finish;
+    iface->set_current_bands = modem_set_current_bands;
+    iface->set_current_bands_finish = modem_set_current_bands_finish;
     iface->create_bearer = modem_create_bearer;
     iface->create_bearer_finish = modem_create_bearer_finish;
 }
