@@ -40,6 +40,7 @@ struct _MMBroadbandBearerHsoPrivate {
     gpointer connect_pending;
     guint connect_pending_id;
     gulong connect_cancellable_id;
+    gulong connect_port_closed_id;
 };
 
 /*****************************************************************************/
@@ -53,28 +54,6 @@ typedef struct {
     GSimpleAsyncResult *result;
 } GetIpConfig3gppContext;
 
-static GetIpConfig3gppContext *
-get_ip_config_3gpp_context_new (MMBroadbandBearerHso *self,
-                                MMBaseModem *modem,
-                                MMAtSerialPort *primary,
-                                guint cid,
-                                GAsyncReadyCallback callback,
-                                gpointer user_data)
-{
-    GetIpConfig3gppContext *ctx;
-
-    ctx = g_new0 (GetIpConfig3gppContext, 1);
-    ctx->self = g_object_ref (self);
-    ctx->modem = g_object_ref (modem);
-    ctx->primary = g_object_ref (primary);
-    ctx->cid = cid;
-    ctx->result = g_simple_async_result_new (G_OBJECT (self),
-                                             callback,
-                                             user_data,
-                                             get_ip_config_3gpp_context_new);
-    return ctx;
-}
-
 static void
 get_ip_config_context_complete_and_free (GetIpConfig3gppContext *ctx)
 {
@@ -83,7 +62,7 @@ get_ip_config_context_complete_and_free (GetIpConfig3gppContext *ctx)
     g_object_unref (ctx->primary);
     g_object_unref (ctx->modem);
     g_object_unref (ctx->self);
-    g_free (ctx);
+    g_slice_free (GetIpConfig3gppContext, ctx);
 }
 
 static gboolean
@@ -209,8 +188,19 @@ get_ip_config_3gpp (MMBroadbandBearer *self,
                     GAsyncReadyCallback callback,
                     gpointer user_data)
 {
+    GetIpConfig3gppContext *ctx;
     gchar *command;
 
+    ctx = g_slice_new0 (GetIpConfig3gppContext);
+    ctx->self = g_object_ref (self);
+    ctx->modem = g_object_ref (modem);
+    ctx->primary = g_object_ref (primary);
+    ctx->cid = cid;
+    ctx->result = g_simple_async_result_new (G_OBJECT (self),
+                                             callback,
+                                             user_data,
+                                             get_ip_config_3gpp);
+
     command = g_strdup_printf ("AT_OWANDATA=%d", cid);
     mm_base_modem_at_command_full (
         MM_BASE_MODEM (modem),
@@ -221,12 +211,7 @@ get_ip_config_3gpp (MMBroadbandBearer *self,
         FALSE, /* raw */
         NULL, /* cancellable */
         (GAsyncReadyCallback)ip_config_ready,
-        get_ip_config_3gpp_context_new (MM_BROADBAND_BEARER_HSO (self),
-                                        MM_BASE_MODEM (modem),
-                                        primary,
-                                        cid,
-                                        callback,
-                                        user_data));
+        ctx);
     g_free (command);
 }
 
@@ -240,49 +225,23 @@ typedef struct {
     guint cid;
     GCancellable *cancellable;
     GSimpleAsyncResult *result;
+    MMPort *data;
     guint auth_idx;
     GError *saved_error;
 } Dial3gppContext;
 
-static Dial3gppContext *
-dial_3gpp_context_new (MMBroadbandBearerHso *self,
-                       MMBaseModem *modem,
-                       MMAtSerialPort *primary,
-                       guint cid,
-                       GCancellable *cancellable,
-                       GAsyncReadyCallback callback,
-                       gpointer user_data)
-{
-    Dial3gppContext *ctx;
-
-    ctx = g_new0 (Dial3gppContext, 1);
-    ctx->self = g_object_ref (self);
-    ctx->modem = g_object_ref (modem);
-    ctx->primary = g_object_ref (primary);
-    ctx->cid = cid;
-    ctx->result = g_simple_async_result_new (G_OBJECT (self),
-                                             callback,
-                                             user_data,
-                                             dial_3gpp_context_new);
-    ctx->cancellable = g_object_ref (cancellable);
-
-    /* Always start with the index that worked last time
-     * (will be 0 the first time)*/
-    ctx->auth_idx = self->priv->auth_idx;
-
-    return ctx;
-}
-
 static void
 dial_3gpp_context_complete_and_free (Dial3gppContext *ctx)
 {
     g_simple_async_result_complete_in_idle (ctx->result);
+    if (ctx->data)
+        g_object_unref (ctx->data);
     g_object_unref (ctx->cancellable);
     g_object_unref (ctx->result);
     g_object_unref (ctx->primary);
     g_object_unref (ctx->modem);
     g_object_unref (ctx->self);
-    g_free (ctx);
+    g_slice_free (Dial3gppContext, ctx);
 }
 
 static gboolean
@@ -312,12 +271,15 @@ dial_3gpp_context_complete_and_free_if_cancelled (Dial3gppContext *ctx)
     return TRUE;
 }
 
-static gboolean
+static MMPort *
 dial_3gpp_finish (MMBroadbandBearer *self,
                   GAsyncResult *res,
                   GError **error)
 {
-    return !g_simple_async_result_propagate_error (G_SIMPLE_ASYNC_RESULT (res), error);
+    if (g_simple_async_result_propagate_error (G_SIMPLE_ASYNC_RESULT (res), error))
+        return NULL;
+
+    return MM_PORT (g_object_ref (g_simple_async_result_get_op_res_gpointer (G_SIMPLE_ASYNC_RESULT (res))));
 }
 
 static void
@@ -372,6 +334,11 @@ mm_broadband_bearer_hso_report_connection_status (MMBroadbandBearerHso *self,
         self->priv->connect_cancellable_id = 0;
     }
 
+    if (ctx && self->priv->connect_port_closed_id) {
+        g_signal_handler_disconnect (ctx->primary, self->priv->connect_port_closed_id);
+        self->priv->connect_port_closed_id = 0;
+    }
+
     switch (status) {
     case MM_BROADBAND_BEARER_HSO_CONNECTION_STATUS_UNKNOWN:
         break;
@@ -394,7 +361,9 @@ mm_broadband_bearer_hso_report_connection_status (MMBroadbandBearerHso *self,
             return;
         }
 
-        g_simple_async_result_set_op_res_gboolean (ctx->result, TRUE);
+        g_simple_async_result_set_op_res_gpointer (ctx->result,
+                                                   g_object_ref (ctx->data),
+                                                   (GDestroyNotify)g_object_unref);
         dial_3gpp_context_complete_and_free (ctx);
         return;
 
@@ -461,6 +430,12 @@ connect_timed_out_cb (MMBroadbandBearerHso *self)
         self->priv->connect_cancellable_id = 0;
     }
 
+    /* Remove closed port watch, if found */
+    if (ctx && self->priv->connect_port_closed_id) {
+        g_signal_handler_disconnect (ctx->primary, self->priv->connect_port_closed_id);
+        self->priv->connect_port_closed_id = 0;
+    }
+
     /* Cleanup timeout ID */
     self->priv->connect_pending_id = 0;
 
@@ -486,9 +461,6 @@ connect_cancelled_cb (GCancellable *cancellable,
 {
     Dial3gppContext *ctx;
 
-    /* Recover context and remove timeout */
-    ctx = self->priv->connect_pending;
-
     /* Recover context but DON'T remove it from the private info */
     ctx = self->priv->connect_pending;
 
@@ -504,6 +476,16 @@ connect_cancelled_cb (GCancellable *cancellable,
      * attempt to finish */
 }
 
+static void
+forced_close_cb (MMSerialPort *port,
+                 MMBroadbandBearerHso *self)
+{
+    /* Just treat the forced close event as any other unsolicited message */
+    mm_broadband_bearer_hso_report_connection_status (
+        self,
+        MM_BROADBAND_BEARER_HSO_CONNECTION_STATUS_CONNECTION_FAILED);
+}
+
 static void
 activate_ready (MMBaseModem *modem,
                 GAsyncResult *res,
@@ -546,6 +528,12 @@ activate_ready (MMBaseModem *modem,
                                                                 G_CALLBACK (connect_cancelled_cb),
                                                                 self,
                                                                 NULL);
+
+    /* If we get the port closed, we treat as a connect error */
+    self->priv->connect_port_closed_id = g_signal_connect (ctx->primary,
+                                                           "forced-close",
+                                                           G_CALLBACK (forced_close_cb),
+                                                           self);
 }
 
 static void authenticate (Dial3gppContext *ctx);
@@ -688,21 +676,43 @@ static void
 dial_3gpp (MMBroadbandBearer *self,
            MMBaseModem *modem,
            MMAtSerialPort *primary,
-           MMPort *data, /* unused by us */
            guint cid,
            GCancellable *cancellable,
            GAsyncReadyCallback callback,
            gpointer user_data)
 {
+    Dial3gppContext *ctx;
+
     g_assert (primary != NULL);
 
-    authenticate (dial_3gpp_context_new (MM_BROADBAND_BEARER_HSO (self),
-                                         modem,
-                                         primary,
-                                         cid,
-                                         cancellable,
-                                         callback,
-                                         user_data));
+    ctx = g_slice_new0 (Dial3gppContext);
+    ctx->self = g_object_ref (self);
+    ctx->modem = g_object_ref (modem);
+    ctx->primary = g_object_ref (primary);
+    ctx->cid = cid;
+    ctx->result = g_simple_async_result_new (G_OBJECT (self),
+                                             callback,
+                                             user_data,
+                                             dial_3gpp);
+    ctx->cancellable = g_object_ref (cancellable);
+
+    /* Always start with the index that worked last time
+     * (will be 0 the first time)*/
+    ctx->auth_idx = ctx->self->priv->auth_idx;
+
+    /* We need a net data port */
+    ctx->data = mm_base_modem_get_best_data_port (modem, MM_PORT_TYPE_NET);
+    if (!ctx->data) {
+        g_simple_async_result_set_error (
+            ctx->result,
+            MM_CORE_ERROR,
+            MM_CORE_ERROR_NOT_FOUND,
+            "No valid data port found to launch connection");
+        dial_3gpp_context_complete_and_free (ctx);
+        return;
+    }
+
+    authenticate (ctx);
 }
 
 /*****************************************************************************/
