@@ -627,6 +627,27 @@ trace_received (MMGpsSerialPort *port,
                 const gchar *trace,
                 MMIfaceModemLocation *self)
 {
+    /* Helper to debug GPS location related issues. Don't depend on a real GPS
+     * fix for debugging, just use some random values to update */
+#if 0
+    if (g_str_has_prefix (trace, "$GPGGA")) {
+        GString *str;
+        GDateTime *now;
+
+        now = g_date_time_new_now_utc ();
+        str = g_string_new ("");
+        g_string_append_printf (str,
+                                "$GPGGA,%02u%02u%02u,4807.038,N,01131.000,E,1,08,0.9,545.4,M,46.9,M,,*47",
+                                g_date_time_get_hour (now),
+                                g_date_time_get_minute (now),
+                                g_date_time_get_second (now));
+        mm_iface_modem_location_gps_update (self, str->str);
+        g_string_free (str, TRUE);
+        g_date_time_unref (now);
+        return;
+    }
+#endif
+
     mm_iface_modem_location_gps_update (self, trace);
 }
 
