@@ -26,6 +26,7 @@
 #define _LIBMM_INSIDE_MM
 #include <libmm-glib.h>
 
+#include "mm-iface-modem.h"
 #include "mm-bearer-qmi.h"
 #include "mm-modem-helpers-qmi.h"
 #include "mm-serial-enums-types.h"
@@ -47,23 +48,6 @@ struct _MMBearerQmiPrivate {
 /*****************************************************************************/
 /* Connect */
 
-typedef struct {
-    MMPort *data;
-    MMBearerIpConfig *ipv4_config;
-    MMBearerIpConfig *ipv6_config;
-} ConnectResult;
-
-static void
-connect_result_free (ConnectResult *result)
-{
-    if (result->ipv4_config)
-        g_object_unref (result->ipv4_config);
-    if (result->ipv6_config)
-        g_object_unref (result->ipv6_config);
-    g_object_unref (result->data);
-    g_slice_free (ConnectResult, result);
-}
-
 typedef enum {
     CONNECT_STEP_FIRST,
     CONNECT_STEP_OPEN_QMI_PORT,
@@ -129,25 +113,15 @@ connect_context_complete_and_free (ConnectContext *ctx)
     g_slice_free (ConnectContext, ctx);
 }
 
-static gboolean
+static MMBearerConnectResult *
 connect_finish (MMBearer *self,
                 GAsyncResult *res,
-                MMPort **data,
-                MMBearerIpConfig **ipv4_config,
-                MMBearerIpConfig **ipv6_config,
                 GError **error)
 {
-    ConnectResult *result;
-
     if (g_simple_async_result_propagate_error (G_SIMPLE_ASYNC_RESULT (res), error))
-        return FALSE;
+        return NULL;
 
-    result = (ConnectResult *) g_simple_async_result_get_op_res_gpointer (G_SIMPLE_ASYNC_RESULT (res));
-    *data = MM_PORT (g_object_ref (result->data));
-    *ipv4_config = (result->ipv4_config ? g_object_ref (result->ipv4_config) : NULL);
-    *ipv6_config = (result->ipv6_config ? g_object_ref (result->ipv6_config) : NULL);
-
-    return TRUE;
+    return mm_bearer_connect_result_ref (g_simple_async_result_get_op_res_gpointer (G_SIMPLE_ASYNC_RESULT (res)));
 }
 
 static void connect_context_step (ConnectContext *ctx);
@@ -717,34 +691,37 @@ connect_context_step (ConnectContext *ctx)
         return;
     }
 
-    case CONNECT_STEP_GET_CURRENT_SETTINGS: {
-        QmiMessageWdsGetCurrentSettingsInput *input;
-        QmiClientWds *client;
+    case CONNECT_STEP_GET_CURRENT_SETTINGS:
+        /* If one of IPv4 or IPv6 succeeds, get IP configuration */
+        if (ctx->packet_data_handle_ipv4 || ctx->packet_data_handle_ipv6) {
+            QmiMessageWdsGetCurrentSettingsInput *input;
+            QmiClientWds *client;
 
-        if (ctx->running_ipv4)
-            client = ctx->client_ipv4;
-        else if (ctx->running_ipv6)
-            client = ctx->client_ipv6;
-        else
-            g_assert_not_reached ();
+            if (ctx->running_ipv4)
+                client = ctx->client_ipv4;
+            else if (ctx->running_ipv6)
+                client = ctx->client_ipv6;
+            else
+                g_assert_not_reached ();
 
-        mm_dbg ("Getting IP configuration...");
-        input = build_get_current_settings_input (ctx);
-        qmi_client_wds_get_current_settings (client,
-                                             input,
-                                             45,
-                                             ctx->cancellable,
-                                             (GAsyncReadyCallback)get_current_settings_ready,
-                                             ctx);
-        qmi_message_wds_get_current_settings_input_unref (input);
-        return;
-    }
+            mm_dbg ("Getting IP configuration...");
+            input = build_get_current_settings_input (ctx);
+            qmi_client_wds_get_current_settings (client,
+                                                 input,
+                                                 45,
+                                                 ctx->cancellable,
+                                                 (GAsyncReadyCallback)get_current_settings_ready,
+                                                 ctx);
+            qmi_message_wds_get_current_settings_input_unref (input);
+            return;
+        }
+        /* Just fall down */
+        ctx->step++;
 
     case CONNECT_STEP_LAST:
         /* If one of IPv4 or IPv6 succeeds, we're connected */
         if (ctx->packet_data_handle_ipv4 || ctx->packet_data_handle_ipv6) {
             MMBearerIpConfig *config;
-            ConnectResult *result;
 
             /* Port is connected; update the state */
             mm_port_set_connected (MM_PORT (ctx->data), TRUE);
@@ -771,20 +748,15 @@ connect_context_step (ConnectContext *ctx)
             config = mm_bearer_ip_config_new ();
             mm_bearer_ip_config_set_method (config, MM_BEARER_IP_METHOD_DHCP);
 
-            /* Build result */
-            result = g_slice_new0 (ConnectResult);
-            result->data = g_object_ref (ctx->data);
-            if (ctx->packet_data_handle_ipv4)
-                result->ipv4_config = g_object_ref (config);
-            if (ctx->packet_data_handle_ipv6)
-                result->ipv6_config = g_object_ref (config);
-
-            g_object_unref (config);
-
             /* Set operation result */
-            g_simple_async_result_set_op_res_gpointer (ctx->result,
-                                                       result,
-                                                       (GDestroyNotify)connect_result_free);
+            g_simple_async_result_set_op_res_gpointer (
+                ctx->result,
+                mm_bearer_connect_result_new (
+                    ctx->data,
+                    ctx->packet_data_handle_ipv4 ? config : NULL,
+                    ctx->packet_data_handle_ipv6 ? config : NULL),
+                (GDestroyNotify)mm_bearer_connect_result_unref);
+            g_object_unref (config);
         } else {
             GError *error;
 
@@ -817,6 +789,7 @@ _connect (MMBearer *self,
     MMPort *data;
     MMQmiPort *qmi;
     GError *error = NULL;
+    const gchar *apn;
 
     g_object_get (self,
                   MM_BEARER_MODEM, &modem,
@@ -824,7 +797,7 @@ _connect (MMBearer *self,
     g_assert (modem);
 
     /* Grab a data port */
-    data = mm_base_modem_get_best_data_port (modem);
+    data = mm_base_modem_get_best_data_port (modem, MM_PORT_TYPE_NET);
     if (!data) {
         g_simple_async_report_error_in_idle (
             G_OBJECT (self),
@@ -850,6 +823,35 @@ _connect (MMBearer *self,
         return;
     }
 
+    /* Check whether we have an APN */
+    apn = mm_bearer_properties_get_apn (mm_bearer_peek_config (MM_BEARER (self)));
+
+    /* Is this a 3GPP only modem and no APN was given? If so, error */
+    if (mm_iface_modem_is_3gpp_only (MM_IFACE_MODEM (modem)) && !apn) {
+        g_simple_async_report_error_in_idle (
+            G_OBJECT (self),
+            callback,
+            user_data,
+            MM_CORE_ERROR,
+            MM_CORE_ERROR_INVALID_ARGS,
+            "3GPP connection logic requires APN setting");
+        g_object_unref (modem);
+        return;
+    }
+
+    /* Is this a 3GPP2 only modem and APN was given? If so, error */
+    if (mm_iface_modem_is_cdma_only (MM_IFACE_MODEM (modem)) && apn) {
+        g_simple_async_report_error_in_idle (
+            G_OBJECT (self),
+            callback,
+            user_data,
+            MM_CORE_ERROR,
+            MM_CORE_ERROR_INVALID_ARGS,
+            "3GPP2 doesn't support APN setting");
+        g_object_unref (modem);
+        return;
+    }
+
     g_object_unref (modem);
 
     mm_dbg ("Launching connection with QMI port (%s/%s) and data port (%s/%s)",
@@ -875,30 +877,47 @@ _connect (MMBearer *self,
 
     if (properties) {
         MMBearerAllowedAuth auth;
+        MMBearerIpFamily ip_family;
 
         ctx->apn = g_strdup (mm_bearer_properties_get_apn (properties));
         ctx->user = g_strdup (mm_bearer_properties_get_user (properties));
         ctx->password = g_strdup (mm_bearer_properties_get_password (properties));
-        switch (mm_bearer_properties_get_ip_type (properties)) {
-        case MM_BEARER_IP_FAMILY_IPV4:
-            ctx->ipv4 = TRUE;
-            ctx->ipv6 = FALSE;
-            break;
-        case MM_BEARER_IP_FAMILY_IPV6:
-            ctx->ipv4 = FALSE;
-            ctx->ipv6 = TRUE;
-            break;
-        case MM_BEARER_IP_FAMILY_IPV4V6:
-            ctx->ipv4 = TRUE;
-            ctx->ipv6 = TRUE;
-            break;
-        case MM_BEARER_IP_FAMILY_UNKNOWN:
-        default:
-            mm_dbg ("No specific IP family requested, defaulting to IPv4");
+
+        ip_family = mm_bearer_properties_get_ip_type (properties);
+        if (ip_family == MM_BEARER_IP_FAMILY_NONE ||
+            ip_family == MM_BEARER_IP_FAMILY_ANY) {
+            gchar *ip_family_str;
+
+            ip_family = mm_bearer_get_default_ip_family (self);
+            ip_family_str = mm_bearer_ip_family_build_string_from_mask (ip_family);
+            mm_dbg ("No specific IP family requested, defaulting to %s",
+                    ip_family_str);
             ctx->no_ip_family_preference = TRUE;
+            g_free (ip_family_str);
+        }
+
+        if (ip_family & MM_BEARER_IP_FAMILY_IPV4)
             ctx->ipv4 = TRUE;
-            ctx->ipv6 = FALSE;
-            break;
+        if (ip_family & MM_BEARER_IP_FAMILY_IPV6)
+            ctx->ipv6 = TRUE;
+        if (ip_family & MM_BEARER_IP_FAMILY_IPV4V6) {
+            ctx->ipv4 = TRUE;
+            ctx->ipv6 = TRUE;
+        }
+
+        if (!ctx->ipv4 && !ctx->ipv6) {
+            gchar *str;
+
+            str = mm_bearer_ip_family_build_string_from_mask (ip_family);
+            g_simple_async_result_set_error (
+                ctx->result,
+                MM_CORE_ERROR,
+                MM_CORE_ERROR_UNSUPPORTED,
+                "Unsupported IP type requested: '%s'",
+                str);
+            g_free (str);
+            connect_context_complete_and_free (ctx);
+            return;
         }
 
         auth = mm_bearer_properties_get_allowed_auth (properties);
@@ -1071,7 +1090,7 @@ disconnect_context_step (DisconnectContext *ctx)
             ctx->running_ipv6 = FALSE;
             qmi_client_wds_stop_network (ctx->client_ipv4,
                                          input,
-                                         10,
+                                         30,
                                          NULL,
                                          (GAsyncReadyCallback)stop_network_ready,
                                          ctx);
@@ -1092,7 +1111,7 @@ disconnect_context_step (DisconnectContext *ctx)
             ctx->running_ipv6 = TRUE;
             qmi_client_wds_stop_network (ctx->client_ipv6,
                                          input,
-                                         10,
+                                         30,
                                          NULL,
                                          (GAsyncReadyCallback)stop_network_ready,
                                          ctx);
