@@ -13,6 +13,7 @@
  * Copyright (C) 2008 - 2009 Novell, Inc.
  * Copyright (C) 2009 - 2011 Red Hat, Inc.
  * Copyright (C) 2011 Google, Inc.
+ * Copyright (C) 2011 - 2013 Aleksander Morgado <aleksander@gnu.org>
  */
 
 #include <config.h>
@@ -39,6 +40,8 @@
 /* We require up to 20s to get a proper IP when using PPP */
 #define MM_BEARER_IP_TIMEOUT_DEFAULT 20
 
+#define MM_BEARER_DEFERRED_UNREGISTRATION_TIMEOUT 15
+
 G_DEFINE_TYPE (MMBearer, mm_bearer, MM_GDBUS_TYPE_BEARER_SKELETON);
 
 
@@ -56,6 +59,7 @@ enum {
     PROP_MODEM,
     PROP_STATUS,
     PROP_CONFIG,
+    PROP_DEFAULT_IP_FAMILY,
     PROP_LAST
 };
 
@@ -72,6 +76,8 @@ struct _MMBearerPrivate {
     MMBearerStatus status;
     /* Configuration of the bearer */
     MMBearerProperties *config;
+    /* Default IP family of this bearer */
+    MMBearerIpFamily default_ip_family;
 
     /* Cancellable for connect() */
     GCancellable *connect_cancellable;
@@ -79,12 +85,14 @@ struct _MMBearerPrivate {
     gulong disconnect_signal_handler;
 
     /*-- 3GPP specific --*/
+    guint deferred_3gpp_unregistration_id;
     /* Reason if 3GPP connection is forbidden */
     ConnectionForbiddenReason reason_3gpp;
     /* Handler ID for the registration state change signals */
     guint id_3gpp_registration_change;
 
     /*-- CDMA specific --*/
+    guint deferred_cdma_unregistration_id;
     /* Reason if CDMA connection is forbidden */
     ConnectionForbiddenReason reason_cdma;
     /* Handler IDs for the registration state change signals */
@@ -171,6 +179,31 @@ bearer_update_status_connected (MMBearer *self,
 
 /*****************************************************************************/
 
+static void
+reset_deferred_unregistration (MMBearer *self)
+{
+    if (self->priv->deferred_cdma_unregistration_id) {
+        g_source_remove (self->priv->deferred_cdma_unregistration_id);
+        self->priv->deferred_cdma_unregistration_id = 0;
+    }
+
+    if (self->priv->deferred_3gpp_unregistration_id) {
+        g_source_remove (self->priv->deferred_3gpp_unregistration_id);
+        self->priv->deferred_3gpp_unregistration_id = 0;
+    }
+}
+
+static gboolean
+deferred_3gpp_unregistration_cb (MMBearer *self)
+{
+    g_warn_if_fail (self->priv->reason_3gpp == CONNECTION_FORBIDDEN_REASON_UNREGISTERED);
+    self->priv->deferred_3gpp_unregistration_id = 0;
+
+    mm_dbg ("Forcing bearer disconnection, not registered in 3GPP network");
+    mm_bearer_disconnect_force (MM_BEARER (self));
+    return FALSE;
+}
+
 static void
 modem_3gpp_registration_state_changed (MMIfaceModem3gpp *modem,
                                        GParamSpec *pspec,
@@ -184,12 +217,12 @@ modem_3gpp_registration_state_changed (MMIfaceModem3gpp *modem,
 
     switch (state) {
     case MM_MODEM_3GPP_REGISTRATION_STATE_IDLE:
-    case MM_MODEM_3GPP_REGISTRATION_STATE_SEARCHING:
     case MM_MODEM_3GPP_REGISTRATION_STATE_DENIED:
     case MM_MODEM_3GPP_REGISTRATION_STATE_UNKNOWN:
         self->priv->reason_3gpp = CONNECTION_FORBIDDEN_REASON_UNREGISTERED;
         break;
     case MM_MODEM_3GPP_REGISTRATION_STATE_HOME:
+    case MM_MODEM_3GPP_REGISTRATION_STATE_SEARCHING:
         self->priv->reason_3gpp = CONNECTION_FORBIDDEN_REASON_NONE;
         break;
     case MM_MODEM_3GPP_REGISTRATION_STATE_ROAMING:
@@ -200,29 +233,57 @@ modem_3gpp_registration_state_changed (MMIfaceModem3gpp *modem,
         break;
     }
 
-    /* close connection if some reason found */
-
-    if (self->priv->reason_3gpp == CONNECTION_FORBIDDEN_REASON_NONE)
+    /* If no reason to disconnect, or if it's a mixed CDMA+LTE modem without a CDMA reason,
+     * just don't do anything. */
+    if (self->priv->reason_3gpp == CONNECTION_FORBIDDEN_REASON_NONE ||
+        (mm_iface_modem_is_cdma (MM_IFACE_MODEM (modem)) &&
+         self->priv->reason_cdma == CONNECTION_FORBIDDEN_REASON_NONE)) {
+        reset_deferred_unregistration (self);
         return;
-
-    /* For mixed CDMA+LTE modems, don't close the connection if there is no
-     * CDMA reason to do so. */
-    if (mm_iface_modem_is_cdma (MM_IFACE_MODEM (modem)) &&
-        self->priv->reason_cdma == CONNECTION_FORBIDDEN_REASON_NONE)
-        return;
-
-    switch (self->priv->reason_3gpp) {
-    case CONNECTION_FORBIDDEN_REASON_UNREGISTERED:
-        mm_dbg ("Bearer not allowed to connect, not registered in 3GPP network");
-        break;
-    case CONNECTION_FORBIDDEN_REASON_ROAMING:
-        mm_dbg ("Bearer not allowed to connect, registered in roaming 3GPP network");
-        break;
-    default:
-        break;
     }
 
+    /* Modem is roaming and roaming not allowed, report right away */
+    if (self->priv->reason_3gpp == CONNECTION_FORBIDDEN_REASON_ROAMING) {
+        mm_dbg ("Bearer not allowed to connect, registered in roaming 3GPP network");
+        reset_deferred_unregistration (self);
+        mm_bearer_disconnect_force (MM_BEARER (self));
+        return;
+    }
+
+    /* Modem reports being unregistered */
+    if (self->priv->reason_3gpp == CONNECTION_FORBIDDEN_REASON_UNREGISTERED) {
+        /* If there is already a notification pending, just return */
+        if (self->priv->deferred_3gpp_unregistration_id)
+            return;
+
+        /* If the bearer is not connected, report right away */
+        if (self->priv->status != MM_BEARER_STATUS_CONNECTED) {
+            mm_dbg ("Bearer not allowed to connect, not registered in 3GPP network");
+            mm_bearer_disconnect_force (MM_BEARER (self));
+            return;
+        }
+
+        /* Otherwise, setup the new timeout */
+        mm_dbg ("Connected bearer not registered in 3GPP network");
+        self->priv->deferred_3gpp_unregistration_id =
+            g_timeout_add_seconds (MM_BEARER_DEFERRED_UNREGISTRATION_TIMEOUT,
+                                   (GSourceFunc) deferred_3gpp_unregistration_cb,
+                                   self);
+        return;
+    }
+
+    g_assert_not_reached ();
+}
+
+static gboolean
+deferred_cdma_unregistration_cb (MMBearer *self)
+{
+    g_warn_if_fail (self->priv->reason_cdma == CONNECTION_FORBIDDEN_REASON_UNREGISTERED);
+    self->priv->deferred_cdma_unregistration_id = 0;
+
+    mm_dbg ("Forcing bearer disconnection, not registered in CDMA network");
     mm_bearer_disconnect_force (MM_BEARER (self));
+    return FALSE;
 }
 
 static void
@@ -251,29 +312,46 @@ modem_cdma_registration_state_changed (MMIfaceModemCdma *modem,
         self->priv->reason_cdma = CONNECTION_FORBIDDEN_REASON_UNREGISTERED;
     }
 
-    /* close connection if some reason found */
-
-    if (self->priv->reason_cdma == CONNECTION_FORBIDDEN_REASON_NONE)
+    /* If no reason to disconnect, or if it's a mixed CDMA+LTE modem without a 3GPP reason,
+     * just don't do anything. */
+    if (self->priv->reason_cdma == CONNECTION_FORBIDDEN_REASON_NONE ||
+        (mm_iface_modem_is_3gpp (MM_IFACE_MODEM (modem)) &&
+         self->priv->reason_3gpp == CONNECTION_FORBIDDEN_REASON_NONE)) {
+        reset_deferred_unregistration (self);
         return;
-
-    /* For mixed CDMA+LTE modems, don't close the connection if there is no
-     * 3GPP reason to do so. */
-    if (mm_iface_modem_is_3gpp (MM_IFACE_MODEM (modem)) &&
-        self->priv->reason_3gpp == CONNECTION_FORBIDDEN_REASON_NONE)
-        return;
-
-    switch (self->priv->reason_cdma) {
-    case CONNECTION_FORBIDDEN_REASON_UNREGISTERED:
-        mm_dbg ("Bearer not allowed to connect, not registered in CDMA network");
-        break;
-    case CONNECTION_FORBIDDEN_REASON_ROAMING:
-        mm_dbg ("Bearer not allowed to connect, registered in roaming CDMA network");
-        break;
-    default:
-        break;
     }
 
-    mm_bearer_disconnect_force (MM_BEARER (self));
+    /* Modem is roaming and roaming not allowed, report right away */
+    if (self->priv->reason_cdma == CONNECTION_FORBIDDEN_REASON_ROAMING) {
+        mm_dbg ("Bearer not allowed to connect, registered in roaming CDMA network");
+        reset_deferred_unregistration (self);
+        mm_bearer_disconnect_force (MM_BEARER (self));
+        return;
+    }
+
+    /* Modem reports being unregistered */
+    if (self->priv->reason_cdma == CONNECTION_FORBIDDEN_REASON_UNREGISTERED) {
+        /* If there is already a notification pending, just return */
+        if (self->priv->deferred_cdma_unregistration_id)
+            return;
+
+        /* If the bearer is not connected, report right away */
+        if (self->priv->status != MM_BEARER_STATUS_CONNECTED) {
+            mm_dbg ("Bearer not allowed to connect, not registered in CDMA network");
+            mm_bearer_disconnect_force (MM_BEARER (self));
+            return;
+        }
+
+        /* Otherwise, setup the new timeout */
+        mm_dbg ("Connected bearer not registered in CDMA network");
+        self->priv->deferred_cdma_unregistration_id =
+            g_timeout_add_seconds (MM_BEARER_DEFERRED_UNREGISTRATION_TIMEOUT,
+                                   (GSourceFunc) deferred_cdma_unregistration_cb,
+                                   self);
+        return;
+    }
+
+    g_assert_not_reached ();
 }
 
 static void
@@ -320,15 +398,18 @@ reset_signal_handlers (MMBearer *self)
         return;
 
     if (self->priv->id_3gpp_registration_change) {
-        g_signal_handler_disconnect (self->priv->modem, self->priv->id_3gpp_registration_change);
+        if (g_signal_handler_is_connected (self->priv->modem, self->priv->id_3gpp_registration_change))
+            g_signal_handler_disconnect (self->priv->modem, self->priv->id_3gpp_registration_change);
         self->priv->id_3gpp_registration_change = 0;
     }
     if (self->priv->id_cdma1x_registration_change) {
-        g_signal_handler_disconnect (self->priv->modem, self->priv->id_cdma1x_registration_change);
+        if (g_signal_handler_is_connected (self->priv->modem, self->priv->id_cdma1x_registration_change))
+            g_signal_handler_disconnect (self->priv->modem, self->priv->id_cdma1x_registration_change);
         self->priv->id_cdma1x_registration_change = 0;
     }
     if (self->priv->id_evdo_registration_change) {
-        g_signal_handler_disconnect (self->priv->modem, self->priv->id_evdo_registration_change);
+        if (g_signal_handler_is_connected (self->priv->modem, self->priv->id_evdo_registration_change))
+            g_signal_handler_disconnect (self->priv->modem, self->priv->id_evdo_registration_change);
         self->priv->id_evdo_registration_change = 0;
     }
 }
@@ -370,17 +451,11 @@ connect_ready (MMBearer *self,
 {
     GError *error = NULL;
     gboolean launch_disconnect = FALSE;
-    MMPort *data = NULL;
-    MMBearerIpConfig *ipv4_config = NULL;
-    MMBearerIpConfig *ipv6_config = NULL;
+    MMBearerConnectResult *result;
 
     /* NOTE: connect() implementations *MUST* handle cancellations themselves */
-    if (!MM_BEARER_GET_CLASS (self)->connect_finish (self,
-                                                     res,
-                                                     &data,
-                                                     &ipv4_config,
-                                                     &ipv6_config,
-                                                     &error)) {
+    result = MM_BEARER_GET_CLASS (self)->connect_finish (self, res, &error);
+    if (!result) {
         mm_dbg ("Couldn't connect bearer '%s': '%s'",
                 self->priv->path,
                 error->message);
@@ -397,11 +472,7 @@ connect_ready (MMBearer *self,
     /* Handle cancellations detected after successful connection */
     else if (g_cancellable_is_cancelled (self->priv->connect_cancellable)) {
         mm_dbg ("Connected bearer '%s', but need to disconnect", self->priv->path);
-
-        g_clear_object (&data);
-        g_clear_object (&ipv4_config);
-        g_clear_object (&ipv6_config);
-
+        mm_bearer_connect_result_unref (result);
         g_simple_async_result_set_error (
             simple,
             MM_CORE_ERROR,
@@ -413,15 +484,12 @@ connect_ready (MMBearer *self,
         mm_dbg ("Connected bearer '%s'", self->priv->path);
 
         /* Update bearer and interface status */
-        bearer_update_status_connected (self,
-                                        mm_port_get_device (data),
-                                        ipv4_config,
-                                        ipv6_config);
-
-        g_clear_object (&data);
-        g_clear_object (&ipv4_config);
-        g_clear_object (&ipv6_config);
-
+        bearer_update_status_connected (
+            self,
+            mm_port_get_device (mm_bearer_connect_result_peek_data (result)),
+            mm_bearer_connect_result_peek_ipv4_config (result),
+            mm_bearer_connect_result_peek_ipv6_config (result));
+        mm_bearer_connect_result_unref (result);
         g_simple_async_result_set_op_res_gboolean (simple, TRUE);
     }
 
@@ -433,6 +501,7 @@ connect_ready (MMBearer *self,
             NULL);
     }
 
+    g_clear_object (&self->priv->connect_cancellable);
     g_simple_async_result_complete (simple);
     g_object_unref (simple);
 }
@@ -852,6 +921,12 @@ mm_bearer_get_config (MMBearer *self)
             NULL);
 }
 
+MMBearerIpFamily
+mm_bearer_get_default_ip_family (MMBearer *self)
+{
+    return self->priv->default_ip_family;
+}
+
 /*****************************************************************************/
 
 static void
@@ -977,6 +1052,9 @@ set_property (GObject *object,
             g_variant_unref (dictionary);
         break;
     }
+    case PROP_DEFAULT_IP_FAMILY:
+        self->priv->default_ip_family = g_value_get_enum (value);
+        break;
     default:
         G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
         break;
@@ -1007,6 +1085,9 @@ get_property (GObject *object,
     case PROP_CONFIG:
         g_value_set_object (value, self->priv->config);
         break;
+    case PROP_DEFAULT_IP_FAMILY:
+        g_value_set_enum (value, self->priv->default_ip_family);
+        break;
     default:
         G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
         break;
@@ -1023,6 +1104,7 @@ mm_bearer_init (MMBearer *self)
     self->priv->status = MM_BEARER_STATUS_DISCONNECTED;
     self->priv->reason_3gpp = CONNECTION_FORBIDDEN_REASON_NONE;
     self->priv->reason_cdma = CONNECTION_FORBIDDEN_REASON_NONE;
+    self->priv->default_ip_family = MM_BEARER_IP_FAMILY_IPV4;
 
     /* Set defaults */
     mm_gdbus_bearer_set_interface (MM_GDBUS_BEARER (self), NULL);
@@ -1057,6 +1139,7 @@ dispose (GObject *object)
     }
 
     reset_signal_handlers (self);
+    reset_deferred_unregistration (self);
 
     g_clear_object (&self->priv->modem);
     g_clear_object (&self->priv->config);
@@ -1119,4 +1202,82 @@ mm_bearer_class_init (MMBearerClass *klass)
                              MM_TYPE_BEARER_PROPERTIES,
                              G_PARAM_READWRITE);
     g_object_class_install_property (object_class, PROP_CONFIG, properties[PROP_CONFIG]);
+
+    properties[PROP_DEFAULT_IP_FAMILY] =
+        g_param_spec_enum (MM_BEARER_DEFAULT_IP_FAMILY,
+                           "Bearer default IP family",
+                           "IP family to use for this bearer when no IP family is specified",
+                           MM_TYPE_BEARER_IP_FAMILY,
+                           MM_BEARER_IP_FAMILY_IPV4,
+                           G_PARAM_READWRITE);
+    g_object_class_install_property (object_class, PROP_DEFAULT_IP_FAMILY, properties[PROP_DEFAULT_IP_FAMILY]);
+}
+
+/*****************************************************************************/
+/* Helpers to implement connect() */
+
+struct _MMBearerConnectResult {
+    volatile gint ref_count;
+    MMPort *data;
+    MMBearerIpConfig *ipv4_config;
+    MMBearerIpConfig *ipv6_config;
+};
+
+MMBearerConnectResult *
+mm_bearer_connect_result_ref (MMBearerConnectResult *result)
+{
+    g_atomic_int_inc (&result->ref_count);
+    return result;
+}
+
+void
+mm_bearer_connect_result_unref (MMBearerConnectResult *result)
+{
+    if (g_atomic_int_dec_and_test (&result->ref_count)) {
+        if (result->ipv4_config)
+            g_object_unref (result->ipv4_config);
+        if (result->ipv6_config)
+            g_object_unref (result->ipv6_config);
+        if (result->data)
+            g_object_unref (result->data);
+        g_slice_free (MMBearerConnectResult, result);
+    }
+}
+
+MMPort *
+mm_bearer_connect_result_peek_data (MMBearerConnectResult *result)
+{
+    return result->data;
+}
+
+MMBearerIpConfig *
+mm_bearer_connect_result_peek_ipv4_config (MMBearerConnectResult *result)
+{
+    return result->ipv4_config;
+}
+
+MMBearerIpConfig *
+mm_bearer_connect_result_peek_ipv6_config (MMBearerConnectResult *result)
+{
+    return result->ipv6_config;
+}
+
+MMBearerConnectResult *
+mm_bearer_connect_result_new (MMPort *data,
+                              MMBearerIpConfig *ipv4_config,
+                              MMBearerIpConfig *ipv6_config)
+{
+    MMBearerConnectResult *result;
+
+    /* 'data' must always be given */
+    g_assert (MM_IS_PORT (data));
+
+    result = g_slice_new0 (MMBearerConnectResult);
+    result->ref_count = 1;
+    result->data = g_object_ref (data);
+    if (ipv4_config)
+        result->ipv4_config = g_object_ref (ipv4_config);
+    if (ipv6_config)
+        result->ipv6_config = g_object_ref (ipv6_config);
+    return result;
 }
