@@ -72,6 +72,7 @@ mm_iface_modem_3gpp_bind_simple_status (MMIfaceModem3gpp *self,
 typedef struct {
     MMModem3gppRegistrationState cs;
     MMModem3gppRegistrationState ps;
+    MMModem3gppRegistrationState eps;
     gboolean manual_registration;
     GCancellable *pending_registration_cancellable;
     gboolean reloading_operator;
@@ -102,6 +103,7 @@ get_registration_state_context (MMIfaceModem3gpp *self)
         ctx = g_slice_new0 (RegistrationStateContext);
         ctx->cs = MM_MODEM_3GPP_REGISTRATION_STATE_UNKNOWN;
         ctx->ps = MM_MODEM_3GPP_REGISTRATION_STATE_UNKNOWN;
+        ctx->eps = MM_MODEM_3GPP_REGISTRATION_STATE_UNKNOWN;
 
         g_object_set_qdata_full (
             G_OBJECT (self),
@@ -129,12 +131,19 @@ get_consolidated_reg_state (RegistrationStateContext *ctx)
         ctx->ps == MM_MODEM_3GPP_REGISTRATION_STATE_ROAMING)
         return ctx->ps;
 
+    if (ctx->eps == MM_MODEM_3GPP_REGISTRATION_STATE_HOME ||
+        ctx->eps == MM_MODEM_3GPP_REGISTRATION_STATE_ROAMING)
+        return ctx->eps;
+
     if (ctx->cs == MM_MODEM_3GPP_REGISTRATION_STATE_SEARCHING)
         return ctx->cs;
 
     if (ctx->ps == MM_MODEM_3GPP_REGISTRATION_STATE_SEARCHING)
         return ctx->ps;
 
+    if (ctx->eps == MM_MODEM_3GPP_REGISTRATION_STATE_SEARCHING)
+        return ctx->eps;
+
     return ctx->cs;
 }
 
@@ -183,6 +192,7 @@ register_in_network_context_failed (RegisterInNetworkContext *ctx,
 {
     mm_iface_modem_3gpp_update_cs_registration_state (ctx->self, MM_MODEM_3GPP_REGISTRATION_STATE_IDLE);
     mm_iface_modem_3gpp_update_ps_registration_state (ctx->self, MM_MODEM_3GPP_REGISTRATION_STATE_IDLE);
+    mm_iface_modem_3gpp_update_eps_registration_state (ctx->self, MM_MODEM_3GPP_REGISTRATION_STATE_IDLE);
     mm_iface_modem_3gpp_update_access_technologies (ctx->self, MM_MODEM_ACCESS_TECHNOLOGY_UNKNOWN);
     mm_iface_modem_3gpp_update_location (ctx->self, 0, 0);
 
@@ -719,21 +729,25 @@ mm_iface_modem_3gpp_run_registration_checks (MMIfaceModem3gpp *self,
 {
     gboolean cs_supported = FALSE;
     gboolean ps_supported = FALSE;
+    gboolean eps_supported = FALSE;
 
     g_assert (MM_IFACE_MODEM_3GPP_GET_INTERFACE (self)->run_registration_checks != NULL);
 
     g_object_get (self,
                   MM_IFACE_MODEM_3GPP_CS_NETWORK_SUPPORTED, &cs_supported,
                   MM_IFACE_MODEM_3GPP_PS_NETWORK_SUPPORTED, &ps_supported,
+                  MM_IFACE_MODEM_3GPP_EPS_NETWORK_SUPPORTED, &eps_supported,
                   NULL);
 
-    mm_dbg ("Running registration checks (CS: '%s', PS: '%s')",
+    mm_dbg ("Running registration checks (CS: '%s', PS: '%s', EPS: '%s')",
             cs_supported ? "yes" : "no",
-            ps_supported ? "yes" : "no");
+            ps_supported ? "yes" : "no",
+            eps_supported ? "yes" : "no");
 
     MM_IFACE_MODEM_3GPP_GET_INTERFACE (self)->run_registration_checks (self,
                                                                        cs_supported,
                                                                        ps_supported,
+                                                                       eps_supported,
                                                                        callback,
                                                                        user_data);
 }
@@ -942,6 +956,10 @@ mm_iface_modem_3gpp_update_access_technologies (MMIfaceModem3gpp *self,
                                                 MMModemAccessTechnology access_tech)
 {
     MMModem3gppRegistrationState state;
+    RegistrationStateContext *ctx;
+
+    ctx = get_registration_state_context (self);
+    g_assert (ctx);
 
     g_object_get (self,
                   MM_IFACE_MODEM_3GPP_REGISTRATION_STATE, &state,
@@ -950,7 +968,8 @@ mm_iface_modem_3gpp_update_access_technologies (MMIfaceModem3gpp *self,
     /* Even if registration state didn't change, report access technology,
      * but only if something valid to report */
     if (state == MM_MODEM_3GPP_REGISTRATION_STATE_HOME ||
-        state == MM_MODEM_3GPP_REGISTRATION_STATE_ROAMING) {
+        state == MM_MODEM_3GPP_REGISTRATION_STATE_ROAMING ||
+        ctx->reloading_operator) {
         if (access_tech != MM_MODEM_ACCESS_TECHNOLOGY_UNKNOWN)
             mm_iface_modem_update_access_technologies (MM_IFACE_MODEM (self),
                                                        access_tech,
@@ -1018,26 +1037,50 @@ update_registration_reload_current_operator_ready (MMIfaceModem3gpp *self,
     ctx->reloading_operator = FALSE;
 }
 
+static void
+update_non_registered_state (MMIfaceModem3gpp *self,
+                             MMModem3gppRegistrationState old_state,
+                             MMModem3gppRegistrationState new_state)
+{
+    /* Not registered neither in home nor roaming network */
+    mm_iface_modem_3gpp_clear_current_operator (self);
+
+    /* The property in the interface is bound to the property
+     * in the skeleton, so just updating here is enough */
+    g_object_set (self,
+                  MM_IFACE_MODEM_3GPP_REGISTRATION_STATE, new_state,
+                  NULL);
+
+    mm_iface_modem_update_subsystem_state (
+        MM_IFACE_MODEM (self),
+        SUBSYSTEM_3GPP,
+        (new_state == MM_MODEM_3GPP_REGISTRATION_STATE_SEARCHING ?
+         MM_MODEM_STATE_SEARCHING :
+         MM_MODEM_STATE_ENABLED),
+        MM_MODEM_STATE_CHANGE_REASON_UNKNOWN);
+}
+
 static void
 update_registration_state (MMIfaceModem3gpp *self,
-                           MMModem3gppRegistrationState new_state)
+                           MMModem3gppRegistrationState new_state,
+                           gboolean deferrable)
 {
     MMModem3gppRegistrationState old_state = MM_MODEM_3GPP_REGISTRATION_STATE_UNKNOWN;
+    RegistrationStateContext *ctx;
 
     g_object_get (self,
                   MM_IFACE_MODEM_3GPP_REGISTRATION_STATE, &old_state,
                   NULL);
 
+    ctx = get_registration_state_context (self);
+    g_assert (ctx);
+
     /* Only set new state if different */
     if (new_state == old_state)
         return;
 
     if (new_state == MM_MODEM_3GPP_REGISTRATION_STATE_HOME ||
         new_state == MM_MODEM_3GPP_REGISTRATION_STATE_ROAMING) {
-        RegistrationStateContext *ctx;
-
-        ctx = get_registration_state_context (self);
-
         /* If already reloading operator, skip it */
         if (ctx->reloading_operator)
             return;
@@ -1061,22 +1104,7 @@ update_registration_state (MMIfaceModem3gpp *self,
              mm_modem_3gpp_registration_state_get_string (old_state),
              mm_modem_3gpp_registration_state_get_string (new_state));
 
-    /* Not registered neither in home nor roaming network */
-    mm_iface_modem_3gpp_clear_current_operator (self);
-
-    /* The property in the interface is bound to the property
-     * in the skeleton, so just updating here is enough */
-    g_object_set (self,
-                  MM_IFACE_MODEM_3GPP_REGISTRATION_STATE, new_state,
-                  NULL);
-
-    mm_iface_modem_update_subsystem_state (
-        MM_IFACE_MODEM (self),
-        SUBSYSTEM_3GPP,
-        (new_state == MM_MODEM_3GPP_REGISTRATION_STATE_SEARCHING ?
-         MM_MODEM_STATE_SEARCHING :
-         MM_MODEM_STATE_ENABLED),
-        MM_MODEM_STATE_CHANGE_REASON_UNKNOWN);
+    update_non_registered_state (self, old_state, new_state);
 }
 
 void
@@ -1095,7 +1123,7 @@ mm_iface_modem_3gpp_update_cs_registration_state (MMIfaceModem3gpp *self,
 
     ctx = get_registration_state_context (self);
     ctx->cs = state;
-    update_registration_state (self, get_consolidated_reg_state (ctx));
+    update_registration_state (self, get_consolidated_reg_state (ctx), TRUE);
 }
 
 void
@@ -1114,7 +1142,26 @@ mm_iface_modem_3gpp_update_ps_registration_state (MMIfaceModem3gpp *self,
 
     ctx = get_registration_state_context (self);
     ctx->ps = state;
-    update_registration_state (self, get_consolidated_reg_state (ctx));
+    update_registration_state (self, get_consolidated_reg_state (ctx), TRUE);
+}
+
+void
+mm_iface_modem_3gpp_update_eps_registration_state (MMIfaceModem3gpp *self,
+                                                   MMModem3gppRegistrationState state)
+{
+    RegistrationStateContext *ctx;
+    gboolean supported = FALSE;
+
+    g_object_get (self,
+                  MM_IFACE_MODEM_3GPP_EPS_NETWORK_SUPPORTED, &supported,
+                  NULL);
+
+    if (!supported)
+        return;
+
+    ctx = get_registration_state_context (self);
+    ctx->eps = state;
+    update_registration_state (self, get_consolidated_reg_state (ctx), TRUE);
 }
 
 /*****************************************************************************/
@@ -1147,7 +1194,8 @@ periodic_registration_checks_ready (MMIfaceModem3gpp *self,
 
     /* Remove the running tag */
     ctx = g_object_get_qdata (G_OBJECT (self), registration_check_context_quark);
-    ctx->running = FALSE;
+    if (ctx)
+        ctx->running = FALSE;
 }
 
 static gboolean
@@ -1289,6 +1337,7 @@ interface_disabling_step (DisablingContext *ctx)
         ctx->step++;
 
     case DISABLING_STEP_PERIODIC_REGISTRATION_CHECKS:
+        /* Disable periodic registration checks, if they were set */
         periodic_registration_check_disable (ctx->self);
         /* Fall down to next step */
         ctx->step++;
@@ -1296,10 +1345,12 @@ interface_disabling_step (DisablingContext *ctx)
     case DISABLING_STEP_DISABLE_UNSOLICITED_REGISTRATION_EVENTS: {
         gboolean cs_supported = FALSE;
         gboolean ps_supported = FALSE;
+        gboolean eps_supported = FALSE;
 
         g_object_get (ctx->self,
                       MM_IFACE_MODEM_3GPP_CS_NETWORK_SUPPORTED, &cs_supported,
                       MM_IFACE_MODEM_3GPP_PS_NETWORK_SUPPORTED, &ps_supported,
+                      MM_IFACE_MODEM_3GPP_EPS_NETWORK_SUPPORTED, &eps_supported,
                       NULL);
 
         if (MM_IFACE_MODEM_3GPP_GET_INTERFACE (ctx->self)->disable_unsolicited_registration_events &&
@@ -1308,6 +1359,7 @@ interface_disabling_step (DisablingContext *ctx)
                 ctx->self,
                 cs_supported,
                 ps_supported,
+                eps_supported,
                 (GAsyncReadyCallback)disable_unsolicited_registration_events_ready,
                 ctx);
             return;
@@ -1353,7 +1405,7 @@ interface_disabling_step (DisablingContext *ctx)
         ctx->step++;
 
     case DISABLING_STEP_REGISTRATION_STATE:
-        update_registration_state (ctx->self, MM_MODEM_3GPP_REGISTRATION_STATE_UNKNOWN);
+        update_registration_state (ctx->self, MM_MODEM_3GPP_REGISTRATION_STATE_UNKNOWN, FALSE);
         mm_iface_modem_3gpp_update_access_technologies (ctx->self, MM_MODEM_ACCESS_TECHNOLOGY_UNKNOWN);
         mm_iface_modem_3gpp_update_location (ctx->self, 0, 0);
         /* Fall down to next step */
@@ -1617,10 +1669,12 @@ interface_enabling_step (EnablingContext *ctx)
     case ENABLING_STEP_ENABLE_UNSOLICITED_REGISTRATION_EVENTS: {
         gboolean cs_supported = FALSE;
         gboolean ps_supported = FALSE;
+        gboolean eps_supported = FALSE;
 
         g_object_get (ctx->self,
                       MM_IFACE_MODEM_3GPP_CS_NETWORK_SUPPORTED, &cs_supported,
                       MM_IFACE_MODEM_3GPP_PS_NETWORK_SUPPORTED, &ps_supported,
+                      MM_IFACE_MODEM_3GPP_EPS_NETWORK_SUPPORTED, &eps_supported,
                       NULL);
 
         if (MM_IFACE_MODEM_3GPP_GET_INTERFACE (ctx->self)->enable_unsolicited_registration_events &&
@@ -1629,6 +1683,7 @@ interface_enabling_step (EnablingContext *ctx)
                 ctx->self,
                 cs_supported,
                 ps_supported,
+                eps_supported,
                 (GAsyncReadyCallback)enable_unsolicited_registration_events_ready,
                 ctx);
             return;
@@ -1995,6 +2050,23 @@ iface_modem_3gpp_init (gpointer g_iface)
                                TRUE,
                                G_PARAM_READWRITE));
 
+    g_object_interface_install_property
+        (g_iface,
+         g_param_spec_boolean (MM_IFACE_MODEM_3GPP_EPS_NETWORK_SUPPORTED,
+                               "EPS network supported",
+                               "Whether the modem works in the EPS network",
+                               FALSE,
+                               G_PARAM_READWRITE));
+
+    g_object_interface_install_property
+        (g_iface,
+         g_param_spec_flags (MM_IFACE_MODEM_3GPP_IGNORED_FACILITY_LOCKS,
+                             "Ignored locks",
+                             "Ignored facility locks",
+                             MM_TYPE_MODEM_3GPP_FACILITY,
+                             MM_MODEM_3GPP_FACILITY_NONE,
+                             G_PARAM_READWRITE));
+
     initialized = TRUE;
 }
 
