@@ -57,6 +57,7 @@ typedef struct {
     MMPluginManager *self;
     MMDevice *device;
     GSimpleAsyncResult *result;
+    GTimer *timer;
     guint timeout_id;
     gulong grabbed_id;
     gulong released_id;
@@ -101,6 +102,11 @@ find_device_support_context_complete_and_free (FindDeviceSupportContext *ctx)
 {
     g_assert (ctx->timeout_id == 0);
 
+    mm_dbg ("(Plugin Manager) [%s] device support check finished in '%lf' seconds",
+            mm_device_get_path (ctx->device),
+            g_timer_elapsed (ctx->timer, NULL));
+    g_timer_destroy (ctx->timer);
+
     /* Set async operation result */
     if (!mm_device_peek_plugin (ctx->device)) {
         g_simple_async_result_set_error (ctx->result,
@@ -136,40 +142,49 @@ static void
 port_probe_context_finished (PortProbeContext *port_probe_ctx)
 {
     FindDeviceSupportContext *ctx = port_probe_ctx->parent_ctx;
+    MMPlugin *device_plugin;
+
+    /* Get info about the currently scheduled plugin in the device */
+    device_plugin = (MMPlugin *)mm_device_peek_plugin (ctx->device);
 
     if (!port_probe_ctx->best_plugin) {
-        gboolean cancel_remaining;
-        GList *l;
+        /* If the port appeared after an already probed port, which decided that
+         * the Generic plugin was the best one (which is by default not initially
+         * suggested), we'll end up arriving here. Don't ignore it, it may well
+         * be a wwan port that we do need to grab. */
+        if (device_plugin) {
+            mm_dbg ("(Plugin Manager) [%s] assuming port can be handled by the '%s' plugin",
+                    g_udev_device_get_name (port_probe_ctx->port),
+                    mm_plugin_get_name (device_plugin));
+        } else {
+            gboolean cancel_remaining;
+            GList *l;
 
-        mm_dbg ("(Plugin Manager) [%s] not supported by any plugin",
-                g_udev_device_get_name (port_probe_ctx->port));
+            mm_dbg ("(Plugin Manager) [%s] not supported by any plugin",
+                    g_udev_device_get_name (port_probe_ctx->port));
 
-        /* Tell the device to ignore this port */
-        mm_device_ignore_port (ctx->device, port_probe_ctx->port);
+            /* Tell the device to ignore this port */
+            mm_device_ignore_port (ctx->device, port_probe_ctx->port);
 
-        /* If this is the last valid probe which was running (i.e. the last one
-         * not being deferred-until-suggested), cancel all remaining ones. */
-        cancel_remaining = TRUE;
-        for (l = ctx->running_probes; l; l = g_list_next (l)) {
-            PortProbeContext *other = l->data;
+            /* If this is the last valid probe which was running (i.e. the last one
+             * not being deferred-until-suggested), cancel all remaining ones. */
+            cancel_remaining = TRUE;
+            for (l = ctx->running_probes; l; l = g_list_next (l)) {
+                PortProbeContext *other = l->data;
 
-            /* Do not cancel anything if we find at least one probe which is not
-             * waiting for the suggested plugin */
-            if (other != port_probe_ctx && !other->defer_until_suggested) {
-                cancel_remaining = FALSE;
-                break;
+                /* Do not cancel anything if we find at least one probe which is not
+                 * waiting for the suggested plugin */
+                if (other != port_probe_ctx && !other->defer_until_suggested) {
+                    cancel_remaining = FALSE;
+                    break;
+                }
             }
+
+            if (cancel_remaining)
+                /* Set a NULL suggested plugin, will cancel the probes */
+                suggest_port_probe_result (ctx, port_probe_ctx, NULL);
         }
-
-        if (cancel_remaining)
-            /* Set a NULL suggested plugin, will cancel the probes */
-            suggest_port_probe_result (ctx, port_probe_ctx, NULL);
-
     } else {
-        MMPlugin *device_plugin;
-
-        device_plugin = (MMPlugin *)mm_device_peek_plugin (ctx->device);
-
         /* Notify the plugin to the device, if this is the first port probing
          * result we got.
          * Also, if the previously suggested plugin was the GENERIC one and now
@@ -203,19 +218,42 @@ port_probe_context_finished (PortProbeContext *port_probe_ctx)
     /* Remove us from the list of running probes */
     g_assert (g_list_find (ctx->running_probes, port_probe_ctx) != NULL);
     ctx->running_probes = g_list_remove (ctx->running_probes, port_probe_ctx);
-    port_probe_context_free (port_probe_ctx);
 
     /* If there are running probes around, wait for them to finish */
-    if (ctx->running_probes != NULL)
-        return;
+    if (ctx->running_probes != NULL) {
+        GList *l;
+        GString *s = NULL;
+        guint i = 0;
 
+        for (l = ctx->running_probes; l; l = g_list_next (l)) {
+            const gchar *portname = g_udev_device_get_name (((PortProbeContext *)l->data)->port);
+
+            if (!s)
+                s = g_string_new (portname);
+            else
+                g_string_append_printf (s, ", %s", portname);
+            i++;
+        }
+
+        mm_dbg ("(Plugin Manager) '%s' port probe finished, still %u running probes in this device (%s)",
+                g_udev_device_get_name (port_probe_ctx->port), i, s->str);
+        g_string_free (s, TRUE);
+    }
     /* If we didn't use the minimum probing time, wait for it to finish */
-    if (ctx->timeout_id > 0)
-        return;
+    else if (ctx->timeout_id > 0) {
+        mm_dbg ("(Plugin Manager) '%s' port probe finished, last one in device, "
+                "but minimum probing time not consumed yet ('%lf' seconds elapsed)",
+                g_udev_device_get_name (port_probe_ctx->port),
+                g_timer_elapsed (ctx->timer, NULL));
+    } else {
+        mm_dbg ("(Plugin Manager) '%s' port probe finished, last one in device",
+                g_udev_device_get_name (port_probe_ctx->port));
+        /* If we just finished the last running probe, we can now finish the device
+         * support check */
+        find_device_support_context_complete_and_free (ctx);
+    }
 
-    /* If we just finished the last running probe, we can now finish the device
-     * support check */
-    find_device_support_context_complete_and_free (ctx);
+    port_probe_context_free (port_probe_ctx);
 }
 
 static gboolean
@@ -251,6 +289,10 @@ suggest_port_probe_result (FindDeviceSupportContext *ctx,
             /* If we got a task deferred until a suggestion comes,
              * complete it */
             if (port_probe_ctx->defer_until_suggested) {
+                /* Reset the defer until suggested flag; we consider this
+                 * cancelled probe completed now. */
+                port_probe_ctx->defer_until_suggested = FALSE;
+
                 if (suggested_plugin) {
                     mm_dbg ("(Plugin Manager) (%s) [%s] deferred task completed, got suggested plugin",
                             mm_plugin_get_name (suggested_plugin),
@@ -476,13 +518,13 @@ build_plugins_list (MMPluginManager *self,
     if (self->priv->generic)
         list = g_list_append (list, g_object_ref (self->priv->generic));
 
-    mm_info ("(Plugin Manager) [%s] Found '%u' plugins to try...",
-             g_udev_device_get_name (port),
-             g_list_length (list));
+    mm_dbg ("(Plugin Manager) [%s] Found '%u' plugins to try...",
+            g_udev_device_get_name (port),
+            g_list_length (list));
     for (l = list; l; l = g_list_next (l)) {
-        mm_info ("(Plugin Manager) [%s]   Will try with plugin '%s'",
-                 g_udev_device_get_name (port),
-                 mm_plugin_get_name (MM_PLUGIN (l->data)));
+        mm_dbg ("(Plugin Manager) [%s]   Will try with plugin '%s'",
+                g_udev_device_get_name (port),
+                mm_plugin_get_name (MM_PLUGIN (l->data)));
     }
 
     return list;
@@ -495,7 +537,6 @@ device_port_grabbed_cb (MMDevice *device,
 {
     PortProbeContext *port_probe_ctx;
 
-
     /* Launch probing task on this port with the first plugin of the list */
     port_probe_ctx = g_slice_new0 (PortProbeContext);
     port_probe_ctx->parent_ctx = ctx;
@@ -505,13 +546,19 @@ device_port_grabbed_cb (MMDevice *device,
     port_probe_ctx->plugins = build_plugins_list (ctx->self, device, port);
     port_probe_ctx->current = port_probe_ctx->plugins;
 
-    /* If we got one suggested, it will be the first one */
+    /* If we got one suggested, it will be the first one, unless it is the generic plugin */
     port_probe_ctx->suggested_plugin = (!!mm_device_peek_plugin (device) ?
                                         MM_PLUGIN (mm_device_get_plugin (device)) :
                                         NULL);
-    if (port_probe_ctx->suggested_plugin)
-        port_probe_ctx->current = g_list_find (port_probe_ctx->current,
-                                               port_probe_ctx->suggested_plugin);
+    if (port_probe_ctx->suggested_plugin) {
+        if (g_str_equal (mm_plugin_get_name (port_probe_ctx->suggested_plugin),
+                         MM_PLUGIN_GENERIC_NAME))
+            /* Initially ignore generic plugin suggested */
+            g_clear_object (&port_probe_ctx->suggested_plugin);
+        else
+            port_probe_ctx->current = g_list_find (port_probe_ctx->current,
+                                                   port_probe_ctx->suggested_plugin);
+    }
 
     /* Set as running */
     ctx->running_probes = g_list_prepend (ctx->running_probes, port_probe_ctx);
@@ -534,8 +581,31 @@ min_probing_timeout_cb (FindDeviceSupportContext *ctx)
     ctx->timeout_id = 0;
 
     /* If there are no running probes around, we're free to finish */
-    if (ctx->running_probes == NULL)
+    if (ctx->running_probes == NULL) {
+        mm_dbg ("(Plugin Manager) [%s] Minimum probing time consumed and no more ports to probe",
+                mm_device_get_path (ctx->device));
         find_device_support_context_complete_and_free (ctx);
+    } else {
+        GList *l;
+        gboolean not_deferred = FALSE;
+
+        mm_dbg ("(Plugin Manager) [%s] Minimum probing time consumed",
+                mm_device_get_path (ctx->device));
+
+        /* If all we got were probes with 'deferred_until_suggested', just cancel
+         * the probing. May happen e.g. with just 'net' ports */
+        for (l = ctx->running_probes; l; l = g_list_next (l)) {
+            PortProbeContext *port_probe_ctx = l->data;
+
+            if (!port_probe_ctx->defer_until_suggested) {
+                not_deferred = TRUE;
+                break;
+            }
+        }
+
+        if (!not_deferred)
+            suggest_port_probe_result (ctx, NULL, NULL);
+    }
 
     return FALSE;
 }
@@ -548,6 +618,9 @@ mm_plugin_manager_find_device_support (MMPluginManager *self,
 {
     FindDeviceSupportContext *ctx;
 
+    mm_dbg ("(Plugin Manager) [%s] Checking device support...",
+            mm_device_get_path (device));
+
     ctx = g_slice_new0 (FindDeviceSupportContext);
     ctx->self = g_object_ref (self);
     ctx->device = g_object_ref (device);
@@ -571,6 +644,7 @@ mm_plugin_manager_find_device_support (MMPluginManager *self,
      * bring up ports. Given that we launch this only when the first port of the
      * device has been exposed in udev, this timeout effectively means that we
      * leave up to 2s to the remaining ports to appear. */
+    ctx->timer = g_timer_new ();
     ctx->timeout_id = g_timeout_add_seconds (MIN_PROBING_TIME_SECS,
                                              (GSourceFunc)min_probing_timeout_cb,
                                              ctx);
@@ -683,7 +757,7 @@ load_plugins (MMPluginManager *self,
         if (!plugin)
             continue;
 
-        mm_info ("Loaded plugin '%s'", mm_plugin_get_name (plugin));
+        mm_dbg ("Loaded plugin '%s'", mm_plugin_get_name (plugin));
 
         if (g_str_equal (mm_plugin_get_name (plugin), MM_PLUGIN_GENERIC_NAME))
             /* Generic plugin */
@@ -707,8 +781,8 @@ load_plugins (MMPluginManager *self,
         goto out;
     }
 
-    mm_info ("Successfully loaded %u plugins",
-             g_list_length (self->priv->plugins) + !!self->priv->generic);
+    mm_dbg ("Successfully loaded %u plugins",
+            g_list_length (self->priv->plugins) + !!self->priv->generic);
 
 out:
     if (dir)
