@@ -40,6 +40,10 @@
 #include "mm-qmi-port.h"
 #endif
 
+#if defined WITH_MBIM
+#include "mm-mbim-port.h"
+#endif
+
 /*
  * Steps and flow of the Probing process:
  * ----> AT Serial Open
@@ -52,6 +56,8 @@
  *   |----> QCDM?
  * ----> QMI Device Open
  *   |----> QMI Version Info check
+ * ----> MBIM Device Open
+ *   |----> MBIM capabilities check
  */
 
 G_DEFINE_TYPE (MMPortProbe, mm_port_probe, G_TYPE_OBJECT)
@@ -85,6 +91,8 @@ typedef struct {
     guint64 at_send_delay;
     /* Flag to leave/remove echo in AT responses */
     gboolean at_remove_echo;
+    /* Flag to send line-feed at the end of AT commands */
+    gboolean at_send_lf;
     /* Number of times we tried to open the AT port */
     guint at_open_tries;
     /* Custom initialization setup */
@@ -104,6 +112,11 @@ typedef struct {
     MMQmiPort *qmi_port;
 #endif
 
+#if defined WITH_MBIM
+    /* ---- MBIM probing specific context ---- */
+    MMMbimPort *mbim_port;
+#endif
+
 } PortProbeRunTask;
 
 struct _MMPortProbePrivate {
@@ -119,6 +132,7 @@ struct _MMPortProbePrivate {
     gchar *product;
     gboolean is_icera;
     gboolean is_qmi;
+    gboolean is_mbim;
 
     /* Current probing task. Only one can be available at a time */
     PortProbeRunTask *task;
@@ -136,10 +150,11 @@ mm_port_probe_set_result_at (MMPortProbe *self,
                 g_udev_device_get_subsystem (self->priv->port),
                 g_udev_device_get_name (self->priv->port));
 
-        /* Also set as not a QCDM/QMI port */
+        /* Also set as not a QCDM/QMI/MBIM port */
         self->priv->is_qcdm = FALSE;
         self->priv->is_qmi = FALSE;
-        self->priv->flags |= (MM_PORT_PROBE_QCDM | MM_PORT_PROBE_QMI);
+        self->priv->is_mbim = FALSE;
+        self->priv->flags |= (MM_PORT_PROBE_QCDM | MM_PORT_PROBE_QMI | MM_PORT_PROBE_MBIM);
     } else {
         mm_dbg ("(%s/%s) port is not AT-capable",
                 g_udev_device_get_subsystem (self->priv->port),
@@ -223,9 +238,10 @@ mm_port_probe_set_result_qcdm (MMPortProbe *self,
                 g_udev_device_get_subsystem (self->priv->port),
                 g_udev_device_get_name (self->priv->port));
 
-        /* Also set as not an AT/QMI port */
+        /* Also set as not an AT/QMI/MBIM port */
         self->priv->is_at = FALSE;
         self->priv->is_qmi = FALSE;
+        self->priv->is_mbim = FALSE;
         self->priv->vendor = NULL;
         self->priv->product = NULL;
         self->priv->is_icera = FALSE;
@@ -233,7 +249,8 @@ mm_port_probe_set_result_qcdm (MMPortProbe *self,
                               MM_PORT_PROBE_AT_VENDOR |
                               MM_PORT_PROBE_AT_PRODUCT |
                               MM_PORT_PROBE_AT_ICERA |
-                              MM_PORT_PROBE_QMI);
+                              MM_PORT_PROBE_QMI |
+                              MM_PORT_PROBE_MBIM);
     } else
         mm_dbg ("(%s/%s) port is not QCDM-capable",
                 g_udev_device_get_subsystem (self->priv->port),
@@ -252,22 +269,54 @@ mm_port_probe_set_result_qmi (MMPortProbe *self,
                 g_udev_device_get_subsystem (self->priv->port),
                 g_udev_device_get_name (self->priv->port));
 
-        /* Also set as not an AT/QCDM port */
+        /* Also set as not an AT/QCDM/MBIM port */
         self->priv->is_at = FALSE;
         self->priv->is_qcdm = FALSE;
+        self->priv->is_mbim = FALSE;
         self->priv->vendor = NULL;
         self->priv->product = NULL;
         self->priv->flags |= (MM_PORT_PROBE_AT |
                               MM_PORT_PROBE_AT_VENDOR |
                               MM_PORT_PROBE_AT_PRODUCT |
                               MM_PORT_PROBE_AT_ICERA |
-                              MM_PORT_PROBE_QCDM);
+                              MM_PORT_PROBE_QCDM |
+                              MM_PORT_PROBE_MBIM);
     } else
         mm_dbg ("(%s/%s) port is not QMI-capable",
                 g_udev_device_get_subsystem (self->priv->port),
                 g_udev_device_get_name (self->priv->port));
 }
 
+void
+mm_port_probe_set_result_mbim (MMPortProbe *self,
+                               gboolean mbim)
+{
+    self->priv->is_mbim = mbim;
+    self->priv->flags |= MM_PORT_PROBE_MBIM;
+
+    if (self->priv->is_mbim) {
+        mm_dbg ("(%s/%s) port is MBIM-capable",
+                g_udev_device_get_subsystem (self->priv->port),
+                g_udev_device_get_name (self->priv->port));
+
+        /* Also set as not an AT/QCDM/QMI port */
+        self->priv->is_at = FALSE;
+        self->priv->is_qcdm = FALSE;
+        self->priv->is_qmi = FALSE;
+        self->priv->vendor = NULL;
+        self->priv->product = NULL;
+        self->priv->flags |= (MM_PORT_PROBE_AT |
+                              MM_PORT_PROBE_AT_VENDOR |
+                              MM_PORT_PROBE_AT_PRODUCT |
+                              MM_PORT_PROBE_AT_ICERA |
+                              MM_PORT_PROBE_QCDM |
+                              MM_PORT_PROBE_QMI);
+    } else
+        mm_dbg ("(%s/%s) port is not MBIM-capable",
+                g_udev_device_get_subsystem (self->priv->port),
+                g_udev_device_get_name (self->priv->port));
+}
+
 static gboolean serial_probe_at (MMPortProbe *self);
 static gboolean serial_probe_qcdm (MMPortProbe *self);
 static void serial_probe_schedule (MMPortProbe *self);
@@ -278,10 +327,9 @@ port_probe_run_task_free (PortProbeRunTask *task)
     if (task->source_id)
         g_source_remove (task->source_id);
 
-    if (task->buffer_full_id)
-        g_source_remove (task->buffer_full_id);
-
     if (task->serial) {
+        if (task->buffer_full_id)
+            g_signal_handler_disconnect (task->serial, task->buffer_full_id);
         if (mm_serial_port_is_open (task->serial))
             mm_serial_port_close (task->serial);
         g_object_unref (task->serial);
@@ -295,6 +343,14 @@ port_probe_run_task_free (PortProbeRunTask *task)
     }
 #endif
 
+#if defined WITH_MBIM
+    if (task->mbim_port) {
+        /* We should have closed it cleanly before */
+        g_assert (!mm_mbim_port_is_open (task->mbim_port));
+        g_object_unref (task->mbim_port);
+    }
+#endif
+
     if (task->cancellable)
         g_object_unref (task->cancellable);
     if (task->at_probing_cancellable)
@@ -351,7 +407,9 @@ port_probe_run_is_cancelled (MMPortProbe *self)
 }
 
 /***************************************************************/
-/* QMI */
+/* QMI & MBIM */
+
+static gboolean wdm_probe (MMPortProbe *self);
 
 #if defined WITH_QMI
 
@@ -378,21 +436,17 @@ qmi_port_open_ready (MMQmiPort *qmi_port,
 
     mm_qmi_port_close (qmi_port);
 
-    /* All done! Finish asynchronously */
-    port_probe_run_task_complete (task, TRUE, NULL);
+    /* Keep on */
+    task->source_id = g_idle_add ((GSourceFunc)wdm_probe, self);
 }
 
 #endif /* WITH_QMI */
 
-static gboolean
+static void
 wdm_probe_qmi (MMPortProbe *self)
 {
     PortProbeRunTask *task = self->priv->task;
 
-    /* If already cancelled, do nothing else */
-    if (port_probe_run_is_cancelled (self))
-        return FALSE;
-
 #if defined WITH_QMI
     mm_dbg ("(%s/%s) probing QMI...",
             g_udev_device_get_subsystem (self->priv->port),
@@ -408,8 +462,109 @@ wdm_probe_qmi (MMPortProbe *self)
 #else
     /* If not compiled with QMI support, just assume we won't have any QMI port */
     mm_port_probe_set_result_qmi (self, FALSE);
-    port_probe_run_task_complete (task, TRUE, NULL);
+    task->source_id = g_idle_add ((GSourceFunc)wdm_probe, self);
 #endif /* WITH_QMI */
+}
+
+#if defined WITH_MBIM
+
+static void
+mbim_port_close_ready (MMMbimPort *mbim_port,
+                       GAsyncResult *res,
+                       MMPortProbe *self)
+{
+    PortProbeRunTask *task = self->priv->task;
+
+    mm_mbim_port_close_finish (mbim_port, res, NULL);
+
+    /* Keep on */
+    task->source_id = g_idle_add ((GSourceFunc)wdm_probe, self);
+}
+
+static void
+mbim_port_open_ready (MMMbimPort *mbim_port,
+                      GAsyncResult *res,
+                      MMPortProbe *self)
+{
+    PortProbeRunTask *task = self->priv->task;
+    GError *error = NULL;
+    gboolean is_mbim;
+
+    is_mbim = mm_mbim_port_open_finish (mbim_port, res, &error);
+    if (!is_mbim) {
+        mm_dbg ("(%s/%s) error checking MBIM support: '%s'",
+                g_udev_device_get_subsystem (self->priv->port),
+                g_udev_device_get_name (self->priv->port),
+                error ? error->message : "unknown error");
+        g_clear_error (&error);
+    }
+
+    /* Set probing result */
+    mm_port_probe_set_result_mbim (self, is_mbim);
+
+    mm_mbim_port_close (task->mbim_port,
+                        (GAsyncReadyCallback)mbim_port_close_ready,
+                        self);
+}
+
+static gboolean
+mbim_sleep_ready (MMPortProbe *self)
+{
+    PortProbeRunTask *task = self->priv->task;
+
+    task->source_id = 0;
+
+    /* Create a port and try to open it */
+    task->mbim_port = mm_mbim_port_new (g_udev_device_get_name (self->priv->port));
+    mm_mbim_port_open (task->mbim_port,
+                       NULL,
+                       (GAsyncReadyCallback)mbim_port_open_ready,
+                       self);
+    return FALSE;
+}
+
+#endif /* WITH_MBIM */
+
+static void
+wdm_probe_mbim (MMPortProbe *self)
+{
+    PortProbeRunTask *task = self->priv->task;
+
+#if defined WITH_MBIM
+    mm_dbg ("(%s/%s) probing MBIM...",
+            g_udev_device_get_subsystem (self->priv->port),
+            g_udev_device_get_name (self->priv->port));
+
+    task->source_id = g_timeout_add_seconds (3, (GSourceFunc)mbim_sleep_ready, self);
+#else
+    /* If not compiled with MBIM support, just assume we won't have any MBIM port */
+    mm_port_probe_set_result_mbim (self, FALSE);
+    task->source_id = g_idle_add ((GSourceFunc)wdm_probe, self);
+#endif /* WITH_MBIM */
+}
+
+static gboolean
+wdm_probe (MMPortProbe *self)
+{
+    PortProbeRunTask *task = self->priv->task;
+
+    task->source_id = 0;
+
+    /* If already cancelled, do nothing else */
+    if (port_probe_run_is_cancelled (self))
+        return FALSE;
+
+    if ((task->flags & MM_PORT_PROBE_QMI) &&
+        !(self->priv->flags & MM_PORT_PROBE_QMI))
+        /* QMI probing needed */
+        wdm_probe_qmi (self);
+    else if ((task->flags & MM_PORT_PROBE_MBIM) &&
+             !(self->priv->flags & MM_PORT_PROBE_MBIM))
+        /* MBIM probing needed */
+        wdm_probe_mbim (self);
+    else
+        /* All done now */
+        port_probe_run_task_complete (task, TRUE, NULL);
 
     return FALSE;
 }
@@ -582,7 +737,6 @@ static const gchar *non_at_strings[] = {
     "os_logids.h",
     /* Sierra CnS port */
     "NETWORK SERVICE CHANGE",
-    "/SRC/AMSS",
     NULL
 };
 
@@ -595,15 +749,16 @@ static gboolean
 is_non_at_response (const guint8 *data, gsize len)
 {
     const gchar **iter;
-    size_t iter_len;
-    int i;
+    gsize iter_len;
+    gsize i;
 
     /* Some devices (observed on a ZTE branded "QUALCOMM INCORPORATED" model
      * "154") spew NULLs from some ports.
      */
-    if (   (len >= sizeof (zerobuf))
-        && (memcmp (data, zerobuf, sizeof (zerobuf)) == 0))
-        return TRUE;
+    for (i = 0; (len >= sizeof (zerobuf)) && (i < len - sizeof (zerobuf)); i++) {
+        if (!memcmp (&data[i], zerobuf, sizeof (zerobuf)))
+            return TRUE;
+    }
 
     /* Check for a well-known non-AT response.  There are some ports (eg many
      * Icera-based chipsets, Qualcomm Gobi devices before their firmware is
@@ -713,16 +868,6 @@ serial_probe_at_parse_response (MMAtSerialPort *port,
         return;
     }
 
-    /* Early-abort AT probing if we get a response that indicates this is
-     * certainly not an AT-capable port.
-     */
-    if (response && is_non_at_response ((const guint8 *) response->str, response->len)) {
-        task->at_result_processor (self, NULL);
-        mm_port_probe_set_result_at (self, FALSE);
-        serial_probe_schedule (self);
-        return;
-    }
-
     if (!task->at_commands->response_processor (task->at_commands->command,
                                                 response ? response->str : NULL,
                                                 !!task->at_commands[1].command,
@@ -938,13 +1083,36 @@ serial_buffer_full (MMSerialPort *serial,
                     GByteArray *buffer,
                     MMPortProbe *self)
 {
+    PortProbeRunTask *task = self->priv->task;
+
     if (is_non_at_response (buffer->data, buffer->len)) {
-        mm_serial_port_close (serial);
+        mm_dbg ("(%s/%s) serial buffer full",
+                g_udev_device_get_subsystem (self->priv->port),
+                g_udev_device_get_name (self->priv->port));
+        /* Don't explicitly close the AT port, just end the AT probing
+         * (or custom init probing) */
         mm_port_probe_set_result_at (self, FALSE);
-        serial_probe_schedule (self);
+        g_cancellable_cancel (task->at_probing_cancellable);
     }
 }
 
+static gboolean
+serial_parser_filter_cb (gpointer filter,
+                         gpointer user_data,
+                         GString *response,
+                         GError **error)
+{
+    if (is_non_at_response ((const guint8 *) response->str, response->len)) {
+        g_set_error (error,
+                     MM_SERIAL_ERROR,
+                     MM_SERIAL_ERROR_PARSE_FAILED,
+                     "Not an AT response");
+        return FALSE;
+    }
+
+    return TRUE;
+}
+
 static gboolean
 serial_open_at (MMPortProbe *self)
 {
@@ -959,6 +1127,8 @@ serial_open_at (MMPortProbe *self)
 
     /* Create AT serial port if not done before */
     if (!task->serial) {
+        gpointer parser;
+
         task->serial = MM_SERIAL_PORT (mm_at_serial_port_new (g_udev_device_get_name (self->priv->port)));
         if (!task->serial) {
             port_probe_run_task_complete (
@@ -975,13 +1145,18 @@ serial_open_at (MMPortProbe *self)
         g_object_set (task->serial,
                       MM_SERIAL_PORT_SEND_DELAY,     task->at_send_delay,
                       MM_AT_SERIAL_PORT_REMOVE_ECHO, task->at_remove_echo,
+                      MM_AT_SERIAL_PORT_SEND_LF,     task->at_send_lf,
                       MM_PORT_CARRIER_DETECT,        FALSE,
                       MM_SERIAL_PORT_SPEW_CONTROL,   TRUE,
                       NULL);
 
+        parser = mm_serial_parser_v1_new ();
+        mm_serial_parser_v1_add_filter (parser,
+                                        serial_parser_filter_cb,
+                                        NULL);
         mm_at_serial_port_set_response_parser (MM_AT_SERIAL_PORT (task->serial),
                                                mm_serial_parser_v1_parse,
-                                               mm_serial_parser_v1_new (),
+                                               parser,
                                                mm_serial_parser_v1_destroy);
     }
 
@@ -1096,6 +1271,7 @@ mm_port_probe_run (MMPortProbe *self,
                    MMPortProbeFlag flags,
                    guint64 at_send_delay,
                    gboolean at_remove_echo,
+                   gboolean at_send_lf,
                    const MMPortProbeAtCommand *at_custom_probe,
                    const MMAsyncMethod *at_custom_init,
                    GAsyncReadyCallback callback,
@@ -1115,6 +1291,7 @@ mm_port_probe_run (MMPortProbe *self,
     task = g_new0 (PortProbeRunTask, 1);
     task->at_send_delay = at_send_delay;
     task->at_remove_echo = at_remove_echo;
+    task->at_send_lf = at_send_lf;
     task->flags = MM_PORT_PROBE_NONE;
     task->at_custom_probe = at_custom_probe;
     task->at_custom_init = at_custom_init ? (MMPortProbeAtCustomInit)at_custom_init->async : NULL;
@@ -1128,7 +1305,7 @@ mm_port_probe_run (MMPortProbe *self,
     /* Check if we already have the requested probing results.
      * We will fix here the 'task->flags' so that we only request probing
      * for the missing things. */
-    for (i = MM_PORT_PROBE_AT; i <= MM_PORT_PROBE_QMI; i = (i << 1)) {
+    for (i = MM_PORT_PROBE_AT; i <= MM_PORT_PROBE_MBIM; i = (i << 1)) {
         if ((flags & i) && !(self->priv->flags & i)) {
             task->flags += i;
         }
@@ -1148,10 +1325,10 @@ mm_port_probe_run (MMPortProbe *self,
     task->cancellable = g_cancellable_new ();
 
     probe_list_str = mm_port_probe_flag_build_string_from_mask (task->flags);
-    mm_info ("(%s/%s) launching port probing: '%s'",
-             g_udev_device_get_subsystem (self->priv->port),
-             g_udev_device_get_name (self->priv->port),
-             probe_list_str);
+    mm_dbg ("(%s/%s) launching port probing: '%s'",
+            g_udev_device_get_subsystem (self->priv->port),
+            g_udev_device_get_name (self->priv->port),
+            probe_list_str);
     g_free (probe_list_str);
 
     /* If any AT probing is needed, start by opening as AT port */
@@ -1170,9 +1347,9 @@ mm_port_probe_run (MMPortProbe *self,
         return;
     }
 
-    /* If QMI probing needed, start by opening as a QMI port */
-    if (task->flags & MM_PORT_PROBE_QMI) {
-        task->source_id = g_idle_add ((GSourceFunc)wdm_probe_qmi, self);
+    /* If QMI/MBIM probing needed, go on */
+    if (task->flags & MM_PORT_PROBE_QMI || task->flags & MM_PORT_PROBE_MBIM) {
+        task->source_id = g_idle_add ((GSourceFunc)wdm_probe, self);
         return;
     }
 
@@ -1266,6 +1443,37 @@ mm_port_probe_list_has_qmi_port (GList *list)
     return FALSE;
 }
 
+gboolean
+mm_port_probe_is_mbim (MMPortProbe *self)
+{
+    const gchar *subsys;
+    const gchar *name;
+
+    g_return_val_if_fail (MM_IS_PORT_PROBE (self), FALSE);
+
+    subsys = g_udev_device_get_subsystem (self->priv->port);
+    name = g_udev_device_get_name (self->priv->port);
+    if (!g_str_has_prefix (subsys, "usb") ||
+        !name ||
+        !g_str_has_prefix (name, "cdc-wdm"))
+        return FALSE;
+
+    return self->priv->is_mbim;
+}
+
+gboolean
+mm_port_probe_list_has_mbim_port (GList *list)
+{
+    GList *l;
+
+    for (l = list; l; l = g_list_next (l)) {
+        if (mm_port_probe_is_mbim (MM_PORT_PROBE (l->data)))
+            return TRUE;
+    }
+
+    return FALSE;
+}
+
 MMPortType
 mm_port_probe_get_port_type (MMPortProbe *self)
 {
@@ -1287,6 +1495,13 @@ mm_port_probe_get_port_type (MMPortProbe *self)
         return MM_PORT_TYPE_QMI;
 #endif
 
+#if defined WITH_MBIM
+    if (g_str_has_prefix (subsys, "usb") &&
+        g_str_has_prefix (name, "cdc-wdm") &&
+        self->priv->is_mbim)
+        return MM_PORT_TYPE_MBIM;
+#endif
+
     if (self->priv->flags & MM_PORT_PROBE_QCDM &&
         self->priv->is_qcdm)
         return MM_PORT_TYPE_QCDM;
