@@ -93,7 +93,11 @@ typedef struct {
     GRegex *regex_ezx_error;
     GRegex *regex_unknown_error;
     GRegex *regex_connect_failed;
+    GRegex *regex_na;
     GRegex *regex_custom_error;
+    /* User-provided parser filter */
+    mm_serial_parser_v1_filter_fn filter_callback;
+    gpointer                      filter_user_data;
 } MMSerialParserV1;
 
 gpointer
@@ -114,9 +118,13 @@ mm_serial_parser_v1_new (void)
     parser->regex_ezx_error = g_regex_new ("\\r\\n\\MODEM ERROR:\\s*(\\d+)\\r\\n$", flags, 0, NULL);
     parser->regex_unknown_error = g_regex_new ("\\r\\n(ERROR)|(COMMAND NOT SUPPORT)\\r\\n$", flags, 0, NULL);
     parser->regex_connect_failed = g_regex_new ("\\r\\n(NO CARRIER)|(BUSY)|(NO ANSWER)|(NO DIALTONE)\\r\\n$", flags, 0, NULL);
+    /* Samsung Z810 may reply "NA" to report a not-available error */
+    parser->regex_na = g_regex_new ("\\r\\nNA\\r\\n", flags, 0, NULL);
 
     parser->regex_custom_successful = NULL;
     parser->regex_custom_error = NULL;
+    parser->filter_callback = NULL;
+    parser->filter_user_data = NULL;
 
     return parser;
 }
@@ -139,6 +147,19 @@ mm_serial_parser_v1_set_custom_regex (gpointer data,
     parser->regex_custom_error = error ? g_regex_ref (error) : NULL;
 }
 
+void
+mm_serial_parser_v1_add_filter (gpointer data,
+                                mm_serial_parser_v1_filter_fn callback,
+                                gpointer user_data)
+{
+    MMSerialParserV1 *parser = (MMSerialParserV1 *) data;
+
+    g_return_if_fail (parser != NULL);
+
+    parser->filter_callback = callback;
+    parser->filter_user_data = user_data;
+}
+
 gboolean
 mm_serial_parser_v1_parse (gpointer data,
                            GString *response,
@@ -160,7 +181,20 @@ mm_serial_parser_v1_parse (gpointer data,
     if (G_UNLIKELY (!response->len))
         return FALSE;
 
-    /* First, check for successful responses */
+    /* First, apply custom filter if any */
+    if (parser->filter_callback &&
+        !parser->filter_callback (parser,
+                                  parser->filter_user_data,
+                                  response,
+                                  &local_error)) {
+        g_assert (local_error != NULL);
+        mm_dbg ("Got response filtered in serial port: %s", local_error->message);
+        g_propagate_error (error, local_error);
+        response_clean (response);
+        return TRUE;
+    }
+
+    /* Then, check for successful responses */
 
     /* Custom successful replies first, if any */
     if (parser->regex_custom_successful) {
@@ -304,6 +338,19 @@ mm_serial_parser_v1_parse (gpointer data,
         }
 
         local_error = mm_connection_error_for_code (code);
+        goto done;
+    }
+
+    /* NA error */
+    found = g_regex_match_full (parser->regex_na,
+                                response->str, response->len,
+                                0, 0, &match_info, NULL);
+    if (found) {
+        /* Assume NA means 'Not Allowed' :) */
+        local_error = g_error_new (MM_MOBILE_EQUIPMENT_ERROR,
+                                   MM_MOBILE_EQUIPMENT_ERROR_NOT_ALLOWED,
+                                   "Not Allowed");
+        goto done;
     }
 
 done:
@@ -326,7 +373,8 @@ mm_serial_parser_v1_is_known_error (const GError *error)
     /* Need to return TRUE for the kind of errors that this parser may set */
     return (error->domain == MM_MOBILE_EQUIPMENT_ERROR ||
             error->domain == MM_CONNECTION_ERROR ||
-            error->domain == MM_MESSAGE_ERROR);
+            error->domain == MM_MESSAGE_ERROR ||
+            g_error_matches (error, MM_SERIAL_ERROR, MM_SERIAL_ERROR_PARSE_FAILED));
 }
 
 void
@@ -346,6 +394,7 @@ mm_serial_parser_v1_destroy (gpointer data)
     g_regex_unref (parser->regex_ezx_error);
     g_regex_unref (parser->regex_unknown_error);
     g_regex_unref (parser->regex_connect_failed);
+    g_regex_unref (parser->regex_na);
 
     if (parser->regex_custom_successful)
         g_regex_unref (parser->regex_custom_successful);
