@@ -57,6 +57,7 @@ enum {
 enum {
     BUFFER_FULL,
     TIMED_OUT,
+    FORCED_CLOSE,
 
     LAST_SIGNAL
 };
@@ -97,6 +98,7 @@ typedef struct {
     guint n_consecutive_timeouts;
 
     guint flash_id;
+    guint reopen_id;
     guint connected_id;
 } MMSerialPortPrivate;
 
@@ -411,7 +413,7 @@ real_config_fd (MMSerialPort *self, int fd, GError **error)
 
     if (memcmp (&stbuf, &other, sizeof (other)) != 0) {
         mm_warn ("(%s): port attributes not fully set",
-                   mm_port_get_device (MM_PORT (self)));
+                 mm_port_get_device (MM_PORT (self)));
     }
 
     return TRUE;
@@ -806,6 +808,27 @@ data_available (GIOChannel *source,
     return TRUE;
 }
 
+static void
+data_watch_enable (MMSerialPort *self, gboolean enable)
+{
+    MMSerialPortPrivate *priv = MM_SERIAL_PORT_GET_PRIVATE (self);
+
+    if (priv->watch_id) {
+        if (enable)
+            g_warn_if_fail (priv->watch_id == 0);
+
+        g_source_remove (priv->watch_id);
+        priv->watch_id = 0;
+    }
+
+    if (enable) {
+        g_return_if_fail (priv->channel != NULL);
+        priv->watch_id = g_io_add_watch (priv->channel,
+                                         G_IO_IN | G_IO_ERR | G_IO_HUP,
+                                         data_available, self);
+    }
+}
+
 static void
 port_connected (MMSerialPort *self, GParamSpec *pspec, gpointer user_data)
 {
@@ -832,6 +855,9 @@ port_connected (MMSerialPort *self, GParamSpec *pspec, gpointer user_data)
             // close the port and error out?
         }
     }
+
+    /* When connected ignore let PPP have all the data */
+    data_watch_enable (self, !connected);
 }
 
 gboolean
@@ -846,15 +872,23 @@ mm_serial_port_open (MMSerialPort *self, GError **error)
     g_return_val_if_fail (MM_IS_SERIAL_PORT (self), FALSE);
 
     priv = MM_SERIAL_PORT_GET_PRIVATE (self);
-
     device = mm_port_get_device (MM_PORT (self));
 
+    if (priv->reopen_id) {
+        g_set_error (error,
+                     MM_SERIAL_ERROR,
+                     MM_SERIAL_ERROR_OPEN_FAILED,
+                     "Could not open serial device %s: reopen operation in progress",
+                     device);
+        return FALSE;
+    }
+
     if (priv->open_count) {
         /* Already open */
         goto success;
     }
 
-    mm_info ("(%s) opening serial port...", device);
+    mm_dbg ("(%s) opening serial port...", device);
 
     g_get_current_time (&tv_start);
 
@@ -913,9 +947,7 @@ mm_serial_port_open (MMSerialPort *self, GError **error)
 
     priv->channel = g_io_channel_unix_new (priv->fd);
     g_io_channel_set_encoding (priv->channel, NULL, NULL);
-    priv->watch_id = g_io_add_watch (priv->channel,
-                                     G_IO_IN | G_IO_ERR | G_IO_HUP,
-                                     data_available, self);
+    data_watch_enable (self, TRUE);
 
     g_warn_if_fail (priv->connected_id == 0);
     priv->connected_id = g_signal_connect (self, "notify::" MM_PORT_CONNECTED,
@@ -924,6 +956,11 @@ mm_serial_port_open (MMSerialPort *self, GError **error)
 success:
     priv->open_count++;
     mm_dbg ("(%s) device open count is %d (open)", device, priv->open_count);
+
+    /* Run additional port config if just opened */
+    if (priv->open_count == 1 && MM_SERIAL_PORT_GET_CLASS (self)->config)
+        MM_SERIAL_PORT_GET_CLASS (self)->config (self);
+
     return TRUE;
 
 error:
@@ -979,7 +1016,7 @@ mm_serial_port_close (MMSerialPort *self)
         GTimeVal tv_start, tv_end;
         struct serial_struct sinfo = { 0 };
 
-        mm_info ("(%s) closing serial port...", device);
+        mm_dbg ("(%s) closing serial port...", device);
 
         mm_port_set_connected (MM_PORT (self), FALSE);
 
@@ -997,8 +1034,7 @@ mm_serial_port_close (MMSerialPort *self)
         g_get_current_time (&tv_start);
 
         if (priv->channel) {
-            g_source_remove (priv->watch_id);
-            priv->watch_id = 0;
+            data_watch_enable (self, FALSE);
             g_io_channel_shutdown (priv->channel, TRUE, NULL);
             g_io_channel_unref (priv->channel);
             priv->channel = NULL;
@@ -1011,7 +1047,7 @@ mm_serial_port_close (MMSerialPort *self)
 
         g_get_current_time (&tv_end);
 
-        mm_info ("(%s) serial port closed", device);
+        mm_dbg ("(%s) serial port closed", device);
 
         /* Some ports don't respond to data and when close is called
          * the serial layer waits up to 30 second (closing_wait) for
@@ -1079,8 +1115,7 @@ mm_serial_port_close_force (MMSerialPort *self)
     if (priv->forced_close)
         return;
 
-    mm_info ("(%s) forced to close port",
-             mm_port_get_device (MM_PORT (self)));
+    mm_dbg ("(%s) forced to close port", mm_port_get_device (MM_PORT (self)));
 
     /* If already closed, done */
     if (!priv->open_count)
@@ -1093,6 +1128,9 @@ mm_serial_port_close_force (MMSerialPort *self)
     /* Mark as having forced the close, so that we don't warn about incorrect
      * open counts */
     priv->forced_close = TRUE;
+
+    /* Notify about the forced close status */
+    g_signal_emit (self, signals[FORCED_CLOSE], 0);
 }
 
 static void
@@ -1175,6 +1213,99 @@ mm_serial_port_queue_command_cached (MMSerialPort *self,
     internal_queue_command (self, command, take_command, TRUE, timeout_seconds, cancellable, callback, user_data);
 }
 
+typedef struct {
+    MMSerialPort *port;
+    guint initial_open_count;
+    MMSerialReopenFn callback;
+    gpointer user_data;
+} ReopenInfo;
+
+static void
+serial_port_reopen_cancel (MMSerialPort *self)
+{
+    MMSerialPortPrivate *priv;
+
+    g_return_if_fail (MM_IS_SERIAL_PORT (self));
+
+    priv = MM_SERIAL_PORT_GET_PRIVATE (self);
+
+    if (priv->reopen_id > 0) {
+        g_source_remove (priv->reopen_id);
+        priv->reopen_id = 0;
+    }
+}
+
+static gboolean
+reopen_do (gpointer data)
+{
+    ReopenInfo *info = (ReopenInfo *) data;
+    MMSerialPortPrivate *priv = MM_SERIAL_PORT_GET_PRIVATE (info->port);
+    GError *error = NULL;
+    guint i;
+
+    priv->reopen_id = 0;
+
+    for (i = 0; i < info->initial_open_count; i++) {
+        if (!mm_serial_port_open (info->port, &error)) {
+            g_prefix_error (&error, "Couldn't reopen port (%u): ", i);
+            break;
+        }
+    }
+
+    info->callback (info->port, error, info->user_data);
+    if (error)
+        g_error_free (error);
+    g_slice_free (ReopenInfo, info);
+    return FALSE;
+}
+
+gboolean
+mm_serial_port_reopen (MMSerialPort *self,
+                       guint32 reopen_time,
+                       MMSerialReopenFn callback,
+                       gpointer user_data)
+{
+    ReopenInfo *info;
+    MMSerialPortPrivate *priv;
+    guint i;
+
+    g_return_val_if_fail (MM_IS_SERIAL_PORT (self), FALSE);
+    priv = MM_SERIAL_PORT_GET_PRIVATE (self);
+
+    if (priv->reopen_id > 0) {
+        GError *error;
+
+        error = g_error_new_literal (MM_CORE_ERROR,
+                                     MM_CORE_ERROR_IN_PROGRESS,
+                                     "Modem is already being reopened.");
+        callback (self, error, user_data);
+        g_error_free (error);
+        return FALSE;
+    }
+
+    info = g_slice_new0 (ReopenInfo);
+    info->port = self;
+    info->callback = callback;
+    info->user_data = user_data;
+
+    priv = MM_SERIAL_PORT_GET_PRIVATE (self);
+    info->initial_open_count = priv->open_count;
+
+    mm_dbg ("(%s) reopening port (%u)",
+            mm_port_get_device (MM_PORT (self)),
+            info->initial_open_count);
+
+    for (i = 0; i < info->initial_open_count; i++)
+        mm_serial_port_close (self);
+
+    if (reopen_time > 0)
+        priv->reopen_id = g_timeout_add (reopen_time, reopen_do, info);
+    else
+        priv->reopen_id = g_idle_add (reopen_do, info);
+
+    return TRUE;
+}
+
 static gboolean
 get_speed (MMSerialPort *self, speed_t *speed, GError **error)
 {
@@ -1544,6 +1675,7 @@ dispose (GObject *object)
 
     mm_serial_port_close_force (MM_SERIAL_PORT (object));
 
+    serial_port_reopen_cancel (MM_SERIAL_PORT (object));
     mm_serial_port_flash_cancel (MM_SERIAL_PORT (object));
 
     G_OBJECT_CLASS (mm_serial_port_parent_class)->dispose (object);
@@ -1646,7 +1778,7 @@ mm_serial_port_class_init (MMSerialPortClass *klass)
     g_object_class_install_property
         (object_class, PROP_FLASH_OK,
          g_param_spec_boolean (MM_SERIAL_PORT_FLASH_OK,
-                               "FlaskOk",
+                               "FlashOk",
                                "Flashing the port (0 baud for a short period) "
                                "is allowed.",
                                TRUE,
@@ -1670,4 +1802,13 @@ mm_serial_port_class_init (MMSerialPortClass *klass)
                       NULL, NULL,
                       g_cclosure_marshal_VOID__UINT,
 					  G_TYPE_NONE, 1, G_TYPE_UINT);
+
+    signals[FORCED_CLOSE] =
+        g_signal_new ("forced-close",
+                  G_OBJECT_CLASS_TYPE (object_class),
+                  G_SIGNAL_RUN_FIRST,
+                  G_STRUCT_OFFSET (MMSerialPortClass, forced_close),
+                  NULL, NULL,
+                  g_cclosure_marshal_VOID__VOID,
+                  G_TYPE_NONE, 0);
 }
